package org.sunflow.core.tesselatable;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.sunflow.math.Matrix4;
import org.sunflow.system.Parser;
import org.sunflow.system.Parser.ParserException;
import org.sunflow.util.FloatArray;

public class Gumbo extends BezierMesh {
    // generate raw patch data from source rib file
    public static void main(String[] args) {
        try {
            Parser p;
            p = new Parser("gumbo.rib");
            int begins = 1;
            System.out.println("{");
            Matrix4 m = Matrix4.IDENTITY;
            p.checkNextToken("AttributeBegin");
            while (begins != 0) {
                if (p.peekNextToken("Patch")) {
                    p.checkNextToken("bicubic");
                    p.checkNextToken("P");
                    float[] patch = parseFloatArray(p);
                    if (patch.length == 48) {
                        // transform patch
                        for (int i = 0; i < 16; i++) {
                            float x = patch[3 * i + 0];
                            float y = patch[3 * i + 1];
                            float z = patch[3 * i + 2];
                            patch[3 * i + 0] = m.transformPX(x, y, z);
                            patch[3 * i + 1] = m.transformPY(x, y, z);
                            patch[3 * i + 2] = m.transformPZ(x, y, z);
                        }
                        System.out.println("{");
                        for (float v : patch)
                            System.out.printf("  %g,\n", v);
                        System.out.println("},");
                    }
                } else if (p.peekNextToken("Translate")) {
                    Matrix4 t = Matrix4.translation(p.getNextFloat(), p.getNextFloat(), p.getNextFloat());
                    m = m.multiply(t);
                } else if (p.peekNextToken("Rotate")) {
                    float angle = (float) Math.toRadians(p.getNextFloat());
                    Matrix4 t = Matrix4.rotate(p.getNextFloat(), p.getNextFloat(), p.getNextFloat(), angle);
                    m = m.multiply(t);
                } else if (p.peekNextToken("Scale")) {
                    Matrix4 t = Matrix4.scale(p.getNextFloat(), p.getNextFloat(), p.getNextFloat());
                    m = m.multiply(t);
                } else if (p.peekNextToken("TransformEnd")) {
                    m = Matrix4.IDENTITY;
                } else if (p.peekNextToken("AttributeBegin")) {
                    begins++;
                } else if (p.peekNextToken("AttributeEnd")) {
                    begins--;
                } else
                    p.getNextToken();
            }
            System.out.println("};");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ParserException e) {
            e.printStackTrace();
        }
    }

    private static float[] parseFloatArray(Parser p) throws IOException {
        FloatArray array = new FloatArray();
        boolean done = false;
        do {
            String s = p.getNextToken();
            if (s.startsWith("["))
                s = s.substring(1);
            if (s.endsWith("]")) {
                s = s.substring(0, s.length() - 1);
                done = true;
            }
            array.add(Float.parseFloat(s));
        } while (!done);
        return array.trim();
    }

    // copy and paste data here
    private static final float[][] PATCHES = {
            { 10.0000f, 2.00000f, 0.00000f, 10.0000f, 2.00000f, 0.00000f,
                    14.0000f, 2.00000f, 0.00000f, 14.0000f, 2.00000f, 0.00000f,
                    10.0000f, 2.00000f, 0.00000f, 10.2277f, 2.22776f,
                    -0.911042f, 13.7722f, 2.22776f, -0.911042f, 14.0000f,
                    2.00000f, 0.00000f, 10.0000f, 6.00000f, 0.00000f, 10.2277f,
                    5.77223f, -0.911041f, 13.7722f, 5.77224f, -0.911041f,
                    14.0000f, 6.00000f, 0.00000f, 10.0000f, 6.00000f, 0.00000f,
                    10.0000f, 6.00000f, 0.00000f, 14.0000f, 6.00000f, 0.00000f,
                    14.0000f, 6.00000f, 0.00000f, },
            { 10.0000f, 2.00000f, 0.00000f, 10.0000f, 2.00000f, 0.00000f,
                    10.2483f, 2.08468f, 5.33563f, 10.0000f, 2.00000f, 8.00000f,
                    10.0000f, 2.00000f, 0.00000f, 10.5077f, 0.457924f,
                    2.06861f, 11.2875f, 1.04546f, 5.33563f, 11.0392f,
                    0.960774f, 8.00000f, 14.0000f, 2.00000f, 0.00000f,
                    13.4933f, 0.456761f, 2.07326f, 12.2875f, 2.17941f,
                    5.23824f, 12.5766f, 1.37605f, 7.76601f, 14.0000f, 2.00000f,
                    0.00000f, 14.0000f, 2.00000f, 0.00000f, 13.5399f, 2.92014f,
                    4.93284f, 14.0000f, 2.00000f, 8.00000f, },
            { 14.0000f, 2.00000f, 0.00000f, 14.0000f, 2.00000f, 0.00000f,
                    13.5399f, 2.92014f, 4.93284f, 14.0000f, 2.00000f, 8.00000f,
                    14.0000f, 2.00000f, 0.00000f, 15.5432f, 2.50660f, 2.07326f,
                    14.6921f, 3.60159f, 4.65188f, 15.8610f, 3.45395f, 6.60420f,
                    14.0000f, 6.00000f, 0.00000f, 15.5425f, 5.49273f, 2.07061f,
                    13.5521f, 3.61407f, 5.16626f, 15.0332f, 3.93355f, 6.96677f,
                    14.0000f, 6.00000f, 0.00000f, 14.0000f, 6.00000f, 0.00000f,
                    12.5859f, 4.58590f, 5.17181f, 14.0000f, 6.00000f, 8.00000f, },
            { 14.0000f, 6.00000f, 0.00000f, 14.0000f, 6.00000f, 0.00000f,
                    12.5859f, 4.58590f, 5.17181f, 14.0000f, 6.00000f, 8.00000f,
                    14.0000f, 6.00000f, 0.00000f, 13.4927f, 7.54257f, 2.07061f,
                    11.6360f, 5.54118f, 5.17727f, 12.2594f, 6.87025f, 7.12974f,
                    10.0000f, 6.00000f, 0.00000f, 10.5090f, 7.54076f, 2.06338f,
                    11.5055f, 6.72455f, 4.92427f, 11.3818f, 7.59478f, 7.30228f,
                    10.0000f, 6.00000f, 0.00000f, 10.0000f, 6.00000f, 0.00000f,
                    10.6165f, 5.55161f, 5.03744f, 10.0000f, 6.00000f, 8.00000f, },
            { 10.0000f, 6.00000f, 0.00000f, 10.0000f, 6.00000f, 0.00000f,
                    10.6165f, 5.55161f, 5.03744f, 10.0000f, 6.00000f, 8.00000f,
                    10.0000f, 6.00000f, 0.00000f, 8.45923f, 5.49092f, 2.06338f,
                    9.71866f, 4.36704f, 5.15174f, 8.96952f, 4.43964f, 7.67212f,
                    10.0000f, 2.00000f, 0.00000f, 8.45792f, 2.50777f, 2.06861f,
                    9.18040f, 3.15264f, 5.33563f, 8.93204f, 3.06795f, 8.00000f,
                    10.0000f, 2.00000f, 0.00000f, 10.0000f, 2.00000f, 0.00000f,
                    10.2483f, 2.08468f, 5.33563f, 10.0000f, 2.00000f, 8.00000f, },
            { 18.0000f, 2.00000f, 0.00000f, 18.0000f, 2.00000f, 0.00000f,
                    22.0000f, 2.00000f, 0.00000f, 22.0000f, 2.00000f, 0.00000f,
                    18.0000f, 2.00000f, 0.00000f, 18.2277f, 2.22776f,
                    -0.911042f, 21.7722f, 2.22776f, -0.911042f, 22.0000f,
                    2.00000f, 0.00000f, 18.0000f, 6.00000f, 0.00000f, 18.2277f,
                    5.77224f, -0.911042f, 21.7722f, 5.77223f, -0.911044f,
                    22.0000f, 6.00000f, 0.00000f, 18.0000f, 6.00000f, 0.00000f,
                    18.0000f, 6.00000f, 0.00000f, 22.0000f, 6.00000f, 0.00000f,
                    22.0000f, 6.00000f, 0.00000f, },
            { 18.0000f, 2.00000f, 0.00000f, 18.0000f, 2.00000f, 0.00000f,
                    18.6124f, 3.22488f, 4.93779f, 18.0000f, 2.00000f, 8.00000f,
                    18.0000f, 2.00000f, 0.00000f, 18.5064f, 0.456647f,
                    2.07371f, 19.8347f, 2.44159f, 5.28208f, 19.5248f, 1.39004f,
                    7.69502f, 22.0000f, 2.00000f, 0.00000f, 21.4928f,
                    0.457306f, 2.07108f, 21.0113f, 1.23344f, 5.33483f,
                    20.9582f, 0.958220f, 8.00000f, 22.0000f, 2.00000f,
                    0.00000f, 22.0000f, 2.00000f, 0.00000f, 22.0531f, 2.27522f,
                    5.33483f, 22.0000f, 2.00000f, 8.00000f, },
            { 22.0000f, 2.00000f, 0.00000f, 22.0000f, 2.00000f, 0.00000f,
                    22.0531f, 2.27522f, 5.33483f, 22.0000f, 2.00000f, 8.00000f,
                    22.0000f, 2.00000f, 0.00000f, 23.5426f, 2.50715f, 2.07108f,
                    23.1026f, 3.32477f, 5.33483f, 23.0495f, 3.04954f, 8.00000f,
                    22.0000f, 6.00000f, 0.00000f, 23.5427f, 5.49294f, 2.07146f,
                    22.2045f, 4.41977f, 5.20618f, 22.8434f, 4.43176f, 7.78914f,
                    22.0000f, 6.00000f, 0.00000f, 22.0000f, 6.00000f, 0.00000f,
                    21.4391f, 5.67950f, 5.02396f, 22.0000f, 6.00000f, 8.00000f, },
            { 22.0000f, 6.00000f, 0.00000f, 22.0000f, 6.00000f, 0.00000f,
                    21.4391f, 5.67950f, 5.02396f, 22.0000f, 6.00000f, 8.00000f,
                    22.0000f, 6.00000f, 0.00000f, 21.4929f, 7.54278f, 2.07146f,
                    20.6640f, 6.95514f, 4.83944f, 20.8048f, 7.75546f, 7.23198f,
                    18.0000f, 6.00000f, 0.00000f, 18.5072f, 7.54257f, 2.07061f,
                    20.3933f, 5.55591f, 5.16253f, 19.7699f, 6.88498f, 7.11501f,
                    18.0000f, 6.00000f, 0.00000f, 18.0000f, 6.00000f, 0.00000f,
                    19.4157f, 4.58430f, 5.16861f, 18.0000f, 6.00000f, 8.00000f, },
            { 18.0000f, 6.00000f, 0.00000f, 18.0000f, 6.00000f, 0.00000f,
                    19.4157f, 4.58430f, 5.16861f, 18.0000f, 6.00000f, 8.00000f,
                    18.0000f, 6.00000f, 0.00000f, 16.4574f, 5.49273f, 2.07061f,
                    18.4561f, 3.63070f, 5.17458f, 16.9751f, 3.95019f, 6.97509f,
                    18.0000f, 2.00000f, 0.00000f, 16.4566f, 2.50649f, 2.07371f,
                    17.5879f, 3.88144f, 4.64919f, 16.4270f, 3.57293f, 6.42706f,
                    18.0000f, 2.00000f, 0.00000f, 18.0000f, 2.00000f, 0.00000f,
                    18.6124f, 3.22488f, 4.93779f, 18.0000f, 2.00000f, 8.00000f, },
            { 18.0000f, 8.00000f, 0.00000f, 18.0000f, 8.00000f, 0.00000f,
                    22.0000f, 8.00000f, 0.00000f, 22.0000f, 8.00000f, 0.00000f,
                    18.0000f, 8.00000f, 0.00000f, 18.2277f, 8.22776f,
                    -0.911043f, 21.7722f, 8.22775f, -0.911043f, 22.0000f,
                    8.00000f, 0.00000f, 18.0000f, 12.0000f, 0.00000f, 18.2277f,
                    11.7722f, -0.911043f, 21.7722f, 11.7722f, -0.911042f,
                    22.0000f, 12.0000f, 0.00000f, 18.0000f, 12.0000f, 0.00000f,
                    18.0000f, 12.0000f, 0.00000f, 22.0000f, 12.0000f, 0.00000f,
                    22.0000f, 12.0000f, 0.00000f, },
            { 18.0000f, 8.00000f, 0.00000f, 18.0000f, 8.00000f, 0.00000f,
                    19.4156f, 9.41569f, 5.16861f, 18.0000f, 8.00000f, 8.00000f,
                    18.0000f, 8.00000f, 0.00000f, 18.5072f, 6.45742f, 2.07061f,
                    20.3933f, 8.44408f, 5.16253f, 19.7699f, 7.11501f, 7.11501f,
                    22.0000f, 8.00000f, 0.00000f, 21.4929f, 6.45721f, 2.07146f,
                    20.6640f, 7.04485f, 4.83943f, 20.8048f, 6.24453f, 7.23198f,
                    22.0000f, 8.00000f, 0.00000f, 22.0000f, 8.00000f, 0.00000f,
                    21.4391f, 8.32049f, 5.02396f, 22.0000f, 8.00000f, 8.00000f, },
            { 22.0000f, 8.00000f, 0.00000f, 22.0000f, 8.00000f, 0.00000f,
                    21.4391f, 8.32049f, 5.02396f, 22.0000f, 8.00000f, 8.00000f,
                    22.0000f, 8.00000f, 0.00000f, 23.5427f, 8.50705f, 2.07146f,
                    22.2045f, 9.58022f, 5.20618f, 22.8434f, 9.56823f, 7.78914f,
                    22.0000f, 12.0000f, 0.00000f, 23.5426f, 11.4928f, 2.07108f,
                    23.1026f, 10.6752f, 5.33483f, 23.0495f, 10.9504f, 8.00000f,
                    22.0000f, 12.0000f, 0.00000f, 22.0000f, 12.0000f, 0.00000f,
                    22.0531f, 11.7247f, 5.33483f, 22.0000f, 12.0000f, 8.00000f, },
            { 22.0000f, 12.0000f, 0.00000f, 22.0000f, 12.0000f, 0.00000f,
                    22.0531f, 11.7247f, 5.33483f, 22.0000f, 12.0000f, 8.00000f,
                    22.0000f, 12.0000f, 0.00000f, 21.4928f, 13.5426f, 2.07108f,
                    21.0113f, 12.7665f, 5.33483f, 20.9582f, 13.0417f, 8.00000f,
                    18.0000f, 12.0000f, 0.00000f, 18.5064f, 13.5433f, 2.07371f,
                    19.8347f, 11.5584f, 5.28208f, 19.5257f, 12.6103f, 7.69484f,
                    18.0000f, 12.0000f, 0.00000f, 18.0000f, 12.0000f, 0.00000f,
                    18.6124f, 10.7751f, 4.93779f, 18.0000f, 12.0000f, 8.00000f, },
            { 18.0000f, 12.0000f, 0.00000f, 18.0000f, 12.0000f, 0.00000f,
                    18.6124f, 10.7751f, 4.93779f, 18.0000f, 12.0000f, 8.00000f,
                    18.0000f, 12.0000f, 0.00000f, 16.4566f, 11.4935f, 2.07371f,
                    17.5879f, 10.1185f, 4.64919f, 16.4281f, 10.4281f, 6.42818f,
                    18.0000f, 8.00000f, 0.00000f, 16.4574f, 8.50726f, 2.07061f,
                    18.4561f, 10.3693f, 5.17458f, 16.9750f, 10.0498f, 6.97509f,
                    18.0000f, 8.00000f, 0.00000f, 18.0000f, 8.00000f, 0.00000f,
                    19.4156f, 9.41569f, 5.16861f, 18.0000f, 8.00000f, 8.00000f, },
            { 10.0000f, 8.00000f, 0.00000f, 10.0000f, 8.00000f, 0.00000f,
                    14.0000f, 8.00000f, 0.00000f, 14.0000f, 8.00000f, 0.00000f,
                    10.0000f, 8.00000f, 0.00000f, 10.2277f, 8.22776f,
                    -0.911042f, 13.7722f, 8.22775f, -0.911042f, 14.0000f,
                    8.00000f, 0.00000f, 10.0000f, 12.0000f, 0.00000f, 10.2277f,
                    11.7722f, -0.911041f, 13.7722f, 11.7722f, -0.911042f,
                    14.0000f, 12.0000f, 0.00000f, 10.0000f, 12.0000f, 0.00000f,
                    10.0000f, 12.0000f, 0.00000f, 14.0000f, 12.0000f, 0.00000f,
                    14.0000f, 12.0000f, 0.00000f, },
            { 10.0000f, 8.00000f, 0.00000f, 10.0000f, 8.00000f, 0.00000f,
                    10.6165f, 8.44838f, 5.03744f, 10.0000f, 8.00000f, 8.00000f,
                    10.0000f, 8.00000f, 0.00000f, 10.5090f, 6.45923f, 2.06338f,
                    11.5055f, 7.27544f, 4.92427f, 11.3818f, 6.40521f, 7.30228f,
                    14.0000f, 8.00000f, 0.00000f, 13.4927f, 6.45742f, 2.07061f,
                    11.6360f, 8.45881f, 5.17727f, 12.2594f, 7.12974f, 7.12974f,
                    14.0000f, 8.00000f, 0.00000f, 14.0000f, 8.00000f, 0.00000f,
                    12.5879f, 9.41203f, 5.17592f, 14.0000f, 8.00000f, 8.00000f, },
            { 14.0000f, 8.00000f, 0.00000f, 14.0000f, 8.00000f, 0.00000f,
                    12.5879f, 9.41203f, 5.17592f, 14.0000f, 8.00000f, 8.00000f,
                    14.0000f, 8.00000f, 0.00000f, 15.5425f, 8.50726f, 2.07061f,
                    13.5438f, 10.3693f, 5.17458f, 15.0249f, 10.0498f, 6.97509f,
                    14.0000f, 12.0000f, 0.00000f, 15.5433f, 11.4935f, 2.07371f,
                    14.4120f, 10.1185f, 4.64919f, 15.5718f, 10.4281f, 6.42818f,
                    14.0000f, 12.0000f, 0.00000f, 14.0000f, 12.0000f, 0.00000f,
                    13.3876f, 10.7753f, 4.93833f, 14.0000f, 12.0000f, 8.00000f, },
            { 14.0000f, 12.0000f, 0.00000f, 14.0000f, 12.0000f, 0.00000f,
                    13.3876f, 10.7753f, 4.93833f, 14.0000f, 12.0000f, 8.00000f,
                    14.0000f, 12.0000f, 0.00000f, 13.4935f, 13.5433f, 2.07371f,
                    12.1678f, 11.5573f, 5.28260f, 12.4764f, 12.6094f, 7.69529f,
                    10.0000f, 12.0000f, 0.00000f, 10.5077f, 13.5420f, 2.06861f,
                    11.2881f, 12.9550f, 5.33563f, 11.0397f, 13.0397f, 7.99999f,
                    10.0000f, 12.0000f, 0.00000f, 10.0000f, 12.0000f, 0.00000f,
                    10.2483f, 11.9153f, 5.33563f, 10.0000f, 12.0000f, 8.00000f, },
            { 10.0000f, 12.0000f, 0.00000f, 10.0000f, 12.0000f, 0.00000f,
                    10.2483f, 11.9153f, 5.33563f, 10.0000f, 12.0000f, 8.00000f,
                    10.0000f, 12.0000f, 0.00000f, 8.45792f, 11.4922f, 2.06861f,
                    9.18040f, 10.8473f, 5.33563f, 8.93204f, 10.9320f, 8.00000f,
                    10.0000f, 8.00000f, 0.00000f, 8.45923f, 8.50907f, 2.06338f,
                    9.71866f, 9.63295f, 5.15174f, 8.96952f, 9.56035f, 7.67212f,
                    10.0000f, 8.00000f, 0.00000f, 10.0000f, 8.00000f, 0.00000f,
                    10.6165f, 8.44838f, 5.03744f, 10.0000f, 8.00000f, 8.00000f, },
            { 14.0000f, 2.00000f, 8.00000f, 15.9088f, 2.83677f, 8.31378f,
                    16.2210f, 2.71156f, 8.35578f, 18.0000f, 2.00000f, 8.00000f,
                    15.8610f, 3.45395f, 6.60420f, 16.3392f, 3.39355f, 7.40290f,
                    15.8223f, 3.41220f, 7.35328f, 16.4270f, 3.57293f, 6.42706f,
                    15.0332f, 3.93355f, 6.96677f, 15.8446f, 4.10859f, 7.95327f,
                    16.1636f, 4.12523f, 7.96159f, 16.9751f, 3.95019f, 6.97509f,
                    14.0000f, 6.00000f, 8.00000f, 15.1986f, 5.40068f, 8.59932f,
                    16.8013f, 5.40068f, 8.59932f, 18.0000f, 6.00000f, 8.00000f, },
            { 14.0000f, 6.00000f, 8.00000f, 15.1986f, 5.40068f, 8.59932f,
                    16.8013f, 5.40068f, 8.59932f, 18.0000f, 6.00000f, 8.00000f,
                    13.7003f, 6.59931f, 8.29966f, 14.8989f, 5.99999f, 8.89898f,
                    17.1010f, 6.00000f, 8.89898f, 18.2996f, 6.59932f, 8.29966f,
                    13.7003f, 7.40068f, 8.29966f, 14.8989f, 8.00000f, 8.89898f,
                    17.1010f, 8.00000f, 8.89897f, 18.2996f, 7.40068f, 8.29966f,
                    14.0000f, 8.00000f, 8.00000f, 15.1986f, 8.59932f, 8.59932f,
                    16.8013f, 8.59931f, 8.59931f, 18.0000f, 8.00000f, 8.00000f, },
            { 14.0000f, 8.00000f, 8.00000f, 15.1986f, 8.59932f, 8.59932f,
                    16.8013f, 8.59931f, 8.59931f, 18.0000f, 8.00000f, 8.00000f,
                    15.0249f, 10.0498f, 6.97509f, 15.8363f, 9.87476f, 7.96159f,
                    16.1636f, 9.87476f, 7.96159f, 16.9750f, 10.0498f, 6.97509f,
                    15.5718f, 10.4281f, 6.42818f, 16.1744f, 10.5890f, 7.35264f,
                    15.8255f, 10.5890f, 7.35264f, 16.4281f, 10.4281f, 6.42818f,
                    14.0000f, 12.0000f, 8.00000f, 15.7780f, 11.2887f, 8.35561f,
                    16.2225f, 11.2890f, 8.35548f, 18.0000f, 12.0000f, 8.00000f, },
            { 10.0000f, 6.00000f, 8.00000f, 11.3818f, 7.59478f, 7.30228f,
                    12.2594f, 6.87025f, 7.12974f, 14.0000f, 6.00000f, 8.00000f,
                    10.6753f, 7.02258f, 8.21487f, 11.3643f, 7.71766f, 7.63807f,
                    12.4302f, 7.23435f, 7.66462f, 13.7003f, 6.59931f, 8.29966f,
                    10.6753f, 6.97741f, 8.21487f, 11.3643f, 6.28233f, 7.63807f,
                    12.4302f, 6.76564f, 7.66462f, 13.7003f, 7.40068f, 8.29966f,
                    10.0000f, 8.00000f, 8.00000f, 11.3818f, 6.40521f, 7.30228f,
                    12.2594f, 7.12974f, 7.12974f, 14.0000f, 8.00000f, 8.00000f, },
            { 18.0000f, 6.00000f, 8.00000f, 19.7699f, 6.88498f, 7.11501f,
                    20.8048f, 7.75546f, 7.23198f, 22.0000f, 6.00000f, 8.00000f,
                    18.2996f, 6.59932f, 8.29966f, 19.5991f, 7.24908f, 7.64989f,
                    20.8231f, 7.85928f, 7.54235f, 21.2738f, 7.35025f, 8.18154f,
                    18.2996f, 7.40068f, 8.29966f, 19.5991f, 6.75091f, 7.64989f,
                    20.8231f, 6.14071f, 7.54235f, 21.2738f, 6.64974f, 8.18154f,
                    18.0000f, 8.00000f, 8.00000f, 19.7699f, 7.11501f, 7.11501f,
                    20.8048f, 6.24453f, 7.23198f, 22.0000f, 8.00000f, 8.00000f, },
            { 10.0000f, 2.00000f, 8.00000f, 9.87124f, 1.95609f, 9.38125f,
                    9.34408f, 1.50592f, 9.94049f, 9.00000f, 1.50000f, 12.0000f,
                    11.0392f, 0.960774f, 8.00000f, 10.9104f, 0.916871f,
                    9.38125f, 10.6451f, 0.577962f, 10.5477f, 10.5822f,
                    0.465660f, 11.9069f, 12.5766f, 1.37605f, 7.76601f,
                    12.7310f, 0.946813f, 9.11659f, 12.3287f, 1.27854f,
                    10.5830f, 12.3287f, 1.27854f, 12.0000f, 14.0000f, 2.00000f,
                    8.00000f, 14.0621f, 1.53028f, 9.33539f, 14.0000f, 1.00000f,
                    10.5830f, 14.0000f, 1.00000f, 12.0000f, },
            { 14.0000f, 2.00000f, 8.00000f, 14.0621f, 1.53028f, 9.33539f,
                    14.0000f, 1.00000f, 10.5830f, 14.0000f, 1.00000f, 12.0000f,
                    15.9088f, 2.83677f, 8.31378f, 15.3278f, 2.08512f, 9.54346f,
                    15.3579f, 0.773669f, 10.5830f, 15.3579f, 0.773669f,
                    12.0000f, 16.2210f, 2.71156f, 8.35578f, 16.7241f, 1.82762f,
                    9.62057f, 16.6232f, 0.00000f, 10.4734f, 16.6232f, 0.00000f,
                    12.0000f, 18.0000f, 2.00000f, 8.00000f, 18.0000f, 1.31729f,
                    9.36540f, 18.0000f, 0.00000f, 10.4734f, 18.0000f, 0.00000f,
                    12.0000f, },
            { 18.0000f, 2.00000f, 8.00000f, 18.0000f, 1.31729f, 9.36540f,
                    18.0000f, 0.00000f, 10.4734f, 18.0000f, 0.00000f, 12.0000f,
                    19.5248f, 1.39004f, 7.69502f, 19.3449f, 0.779322f,
                    9.09642f, 19.4294f, 0.00000f, 10.4734f, 19.4294f, 0.00000f,
                    12.0000f, 20.9582f, 0.958220f, 8.00000f, 20.9309f,
                    0.816847f, 9.36900f, 21.1736f, -0.0212363f, 10.6358f,
                    20.9892f, -0.0107873f, 12.0000f, 22.0000f, 2.00000f,
                    8.00000f, 21.9727f, 1.85862f, 9.36900f, 22.1844f,
                    0.989551f, 10.6358f, 22.0000f, 1.00000f, 12.0000f, },
            { 22.0000f, 2.00000f, 8.00000f, 21.9727f, 1.85862f, 9.36900f,
                    22.1844f, 0.989551f, 10.6358f, 22.0000f, 1.00000f,
                    12.0000f, 23.0495f, 3.04954f, 8.00000f, 23.0222f, 2.90817f,
                    9.36900f, 23.1847f, 1.98990f, 10.6359f, 23.0003f, 2.00035f,
                    12.0000f, 22.8434f, 4.43176f, 7.78914f, 23.1846f, 4.43817f,
                    9.16885f, 22.7037f, 3.28188f, 10.5868f, 22.8245f, 3.59620f,
                    12.0001f, 22.0000f, 6.00000f, 8.00000f, 22.4865f, 5.73632f,
                    9.34339f, 22.8791f, 4.68567f, 10.5866f, 23.0000f, 5.00000f,
                    12.0000f, },
            { 22.0000f, 6.00000f, 8.00000f, 22.4865f, 5.73632f, 9.34339f,
                    22.8791f, 4.68567f, 10.5866f, 23.0000f, 5.00000f, 12.0000f,
                    21.2738f, 7.35025f, 8.18154f, 22.1519f, 6.35839f, 9.42703f,
                    23.0451f, 6.01382f, 10.5865f, 23.1660f, 6.32815f, 11.9998f,
                    21.2738f, 6.64974f, 8.18154f, 22.1519f, 7.64160f, 9.42703f,
                    23.0451f, 7.98617f, 10.5865f, 23.1660f, 7.67185f, 11.9998f,
                    22.0000f, 8.00000f, 8.00000f, 22.4865f, 8.26367f, 9.34339f,
                    22.8791f, 9.31432f, 10.5866f, 23.0000f, 9.00000f, 12.0000f, },
            { 22.0000f, 8.00000f, 8.00000f, 22.4865f, 8.26367f, 9.34339f,
                    22.8791f, 9.31432f, 10.5866f, 23.0000f, 9.00000f, 12.0000f,
                    22.8434f, 9.56823f, 7.78914f, 23.1846f, 9.56182f, 9.16885f,
                    22.7037f, 10.7181f, 10.5868f, 22.8245f, 10.4038f, 12.0001f,
                    23.0495f, 10.9504f, 8.00000f, 23.0222f, 11.0918f, 9.36900f,
                    23.1847f, 12.0100f, 10.6358f, 23.0003f, 11.9996f, 12.0000f,
                    22.0000f, 12.0000f, 8.00000f, 21.9727f, 12.1413f, 9.36900f,
                    22.1844f, 13.0104f, 10.6358f, 22.0000f, 13.0000f, 12.0000f, },
            { 22.0000f, 12.0000f, 8.00000f, 21.9727f, 12.1413f, 9.36900f,
                    22.1844f, 13.0104f, 10.6358f, 22.0000f, 13.0000f, 12.0000f,
                    20.9582f, 13.0417f, 8.00000f, 20.9309f, 13.1831f, 9.36900f,
                    21.1850f, 14.0098f, 10.6358f, 21.0006f, 13.9994f, 12.0000f,
                    19.5257f, 12.6103f, 7.69484f, 19.3449f, 13.2259f, 9.10702f,
                    19.4024f, 13.8246f, 10.4615f, 19.4024f, 13.8246f, 12.0000f,
                    18.0000f, 12.0000f, 8.00000f, 18.0000f, 12.6880f, 9.37601f,
                    18.0000f, 14.0000f, 10.4615f, 18.0000f, 14.0000f, 12.0000f, },
            { 18.0000f, 12.0000f, 8.00000f, 18.0000f, 12.6880f, 9.37601f,
                    18.0000f, 14.0000f, 10.4615f, 18.0000f, 14.0000f, 12.0000f,
                    16.2225f, 11.2890f, 8.35548f, 16.7274f, 12.1789f, 9.63053f,
                    16.6700f, 14.1662f, 10.4615f, 16.6700f, 14.1662f, 12.0000f,
                    15.7780f, 11.2887f, 8.35561f, 15.2726f, 12.1809f, 9.63458f,
                    15.3221f, 14.2203f, 10.4570f, 15.3221f, 14.2203f, 12.0000f,
                    14.0000f, 12.0000f, 8.00000f, 14.0000f, 12.6900f, 9.38006f,
                    14.0000f, 14.0000f, 10.4570f, 14.0000f, 14.0000f, 12.0000f, },
            { 14.0000f, 12.0000f, 8.00000f, 14.0000f, 12.6900f, 9.38006f,
                    14.0000f, 14.0000f, 10.4570f, 14.0000f, 14.0000f, 12.0000f,
                    12.4764f, 12.6094f, 7.69529f, 12.6576f, 13.2269f, 9.11159f,
                    12.2533f, 13.7089f, 10.4570f, 12.2533f, 13.7089f, 12.0000f,
                    11.0397f, 13.0397f, 7.99999f, 10.9112f, 13.0836f, 9.37941f,
                    10.4694f, 13.7456f, 10.6337f, 10.3994f, 13.6615f, 12.0112f,
                    10.0000f, 12.0000f, 8.00000f, 9.87141f, 12.0438f, 9.37941f,
                    9.15043f, 12.6614f, 10.0622f, 9.00000f, 12.5000f, 12.0000f, },
            { 9.00000f, 1.50000f, 12.0000f, 9.07887f, 1.31725f, 13.1541f,
                    9.52420f, 0.976845f, 14.7029f, 10.0000f, 2.00000f,
                    16.0000f, 10.5822f, 0.465660f, 11.9069f, 10.5150f,
                    0.345727f, 13.3585f, 10.9295f, 1.13219f, 14.8938f,
                    11.1033f, 2.00243f, 15.9980f, 12.3287f, 1.27854f, 12.0000f,
                    12.3287f, 1.27854f, 13.4306f, 12.6655f, 0.988352f,
                    14.9883f, 12.6655f, 2.00000f, 16.0000f, 14.0000f, 1.00000f,
                    12.0000f, 14.0000f, 1.00000f, 13.4306f, 14.0000f,
                    0.988352f, 14.9883f, 14.0000f, 2.00000f, 16.0000f, },
            { 14.0000f, 1.00000f, 12.0000f, 14.0000f, 1.00000f, 13.4306f,
                    14.0000f, 0.988352f, 14.9883f, 14.0000f, 2.00000f,
                    16.0000f, 15.3579f, 0.773669f, 12.0000f, 15.3579f,
                    0.773669f, 13.4306f, 15.3354f, 0.988352f, 14.9883f,
                    15.3354f, 2.00000f, 16.0000f, 16.6232f, 0.00000f, 12.0000f,
                    16.6232f, 0.00000f, 13.5696f, 16.8433f, 0.801365f,
                    15.2136f, 16.6794f, 2.01251f, 16.1985f, 18.0000f, 0.00000f,
                    12.0000f, 18.0000f, 0.00000f, 13.5696f, 18.1639f,
                    0.788853f, 15.0150f, 18.0000f, 2.00000f, 16.0000f, },
            { 18.0000f, 0.00000f, 12.0000f, 18.0000f, 0.00000f, 13.5696f,
                    18.1639f, 0.788853f, 15.0150f, 18.0000f, 2.00000f,
                    16.0000f, 19.4294f, 0.00000f, 12.0000f, 19.4294f, 0.00000f,
                    13.5696f, 19.2285f, 0.778765f, 14.8549f, 19.0646f,
                    1.98991f, 15.8399f, 20.9892f, -0.0107873f, 12.0000f,
                    20.8354f, -0.00207507f, 13.1374f, 20.6978f, 0.580195f,
                    14.6361f, 20.2213f, 1.34332f, 15.3489f, 22.0000f, 1.00000f,
                    12.0000f, 21.8462f, 1.00871f, 13.1374f, 21.4764f, 1.23687f,
                    14.2872f, 21.0000f, 2.00000f, 15.0000f, },
            { 22.0000f, 1.00000f, 12.0000f, 21.8462f, 1.00871f, 13.1374f,
                    21.4764f, 1.23687f, 14.2872f, 21.0000f, 2.00000f, 15.0000f,
                    23.0003f, 2.00035f, 12.0000f, 22.8465f, 2.00907f, 13.1374f,
                    22.7558f, 2.31585f, 13.7139f, 22.2794f, 3.07898f, 14.4267f,
                    22.8245f, 3.59620f, 12.0001f, 22.9168f, 3.83627f, 13.0796f,
                    22.3069f, 4.55472f, 14.7895f, 21.9821f, 5.49730f, 15.2768f,
                    23.0000f, 5.00000f, 12.0000f, 23.0922f, 5.24007f, 13.0794f,
                    22.8248f, 6.05742f, 14.0127f, 22.5000f, 7.00000f, 14.5000f, },
            { 22.5000f, 7.00000f, 14.5000f, 22.8248f, 7.94257f, 14.0127f,
                    23.0922f, 8.75992f, 13.0794f, 23.0000f, 9.00000f, 12.0000f,
                    22.5000f, 7.00000f, 14.5000f, 23.1496f, 7.00000f, 13.5254f,
                    23.2583f, 7.43177f, 13.0793f, 23.1660f, 7.67185f, 11.9998f,
                    22.5000f, 7.00000f, 14.5000f, 23.1496f, 7.00000f, 13.5254f,
                    23.2583f, 6.56822f, 13.0793f, 23.1660f, 6.32815f, 11.9998f,
                    22.5000f, 7.00000f, 14.5000f, 22.8248f, 6.05742f, 14.0127f,
                    23.0922f, 5.24007f, 13.0794f, 23.0000f, 5.00000f, 12.0000f, },
            { 23.0000f, 9.00000f, 12.0000f, 23.0922f, 8.75992f, 13.0794f,
                    22.8248f, 7.94257f, 14.0127f, 22.5000f, 7.00000f, 14.5000f,
                    22.8245f, 10.4038f, 12.0001f, 22.9168f, 10.1637f, 13.0796f,
                    22.3069f, 9.44527f, 14.7895f, 21.9821f, 8.50269f, 15.2768f,
                    23.0003f, 11.9996f, 12.0000f, 22.8465f, 11.9909f, 13.1374f,
                    22.7558f, 11.6841f, 13.7139f, 22.2794f, 10.9210f, 14.4267f,
                    22.0000f, 13.0000f, 12.0000f, 21.8462f, 12.9912f, 13.1374f,
                    21.4764f, 12.7631f, 14.2872f, 21.0000f, 12.0000f, 15.0000f, },
            { 22.0000f, 13.0000f, 12.0000f, 21.8462f, 12.9912f, 13.1374f,
                    21.4764f, 12.7631f, 14.2872f, 21.0000f, 12.0000f, 15.0000f,
                    21.0006f, 13.9994f, 12.0000f, 20.8468f, 13.9906f, 13.1374f,
                    20.6978f, 13.4198f, 14.6361f, 20.2213f, 12.6566f, 15.3489f,
                    19.4024f, 13.8246f, 12.0000f, 19.4024f, 13.8246f, 13.5675f,
                    19.2283f, 13.2196f, 14.8562f, 19.0646f, 12.0100f, 15.8399f,
                    18.0000f, 14.0000f, 12.0000f, 18.0000f, 14.0000f, 13.5675f,
                    18.1636f, 13.2095f, 15.0163f, 18.0000f, 12.0000f, 16.0000f, },
            { 18.0000f, 14.0000f, 12.0000f, 18.0000f, 14.0000f, 13.5675f,
                    18.1636f, 13.2095f, 15.0163f, 18.0000f, 12.0000f, 16.0000f,
                    16.6700f, 14.1662f, 12.0000f, 16.6700f, 14.1662f, 13.5675f,
                    16.8420f, 13.1970f, 15.2151f, 16.6783f, 11.9874f, 16.1987f,
                    15.3221f, 14.2203f, 12.0000f, 15.3221f, 14.2203f, 13.5616f,
                    15.3213f, 13.3497f, 14.9846f, 15.3264f, 12.1643f, 16.0012f,
                    14.0000f, 14.0000f, 12.0000f, 14.0000f, 14.0000f, 13.5616f,
                    13.9949f, 13.1854f, 14.9833f, 14.0000f, 12.0000f, 16.0000f, },
            { 14.0000f, 14.0000f, 12.0000f, 14.0000f, 14.0000f, 13.5616f,
                    13.9949f, 13.1854f, 14.9833f, 14.0000f, 12.0000f, 16.0000f,
                    12.2533f, 13.7089f, 12.0000f, 12.2533f, 13.7089f, 13.5616f,
                    12.6271f, 13.0159f, 14.9821f, 12.6322f, 11.8305f, 15.9987f,
                    10.3994f, 13.6615f, 12.0112f, 10.3212f, 13.5676f, 13.5504f,
                    10.9955f, 12.0544f, 14.6564f, 11.1786f, 11.1102f, 15.8546f,
                    9.00000f, 12.5000f, 12.0000f, 8.96405f, 12.4448f, 13.3410f,
                    9.51945f, 12.2157f, 14.4584f, 10.0000f, 11.0000f, 16.0000f, },
            { 10.0000f, 2.00000f, 16.0000f, 10.0036f, 3.39890f, 17.7763f,
                    9.97797f, 5.10529f, 18.0055f, 10.0000f, 7.00000f, 18.0000f,
                    11.1033f, 2.00243f, 15.9980f, 11.3375f, 3.17526f, 17.4863f,
                    11.2748f, 5.10529f, 17.6813f, 11.2968f, 7.00000f, 17.6757f,
                    12.6655f, 2.00000f, 16.0000f, 12.6655f, 3.31963f, 17.3196f,
                    12.6644f, 5.13375f, 18.0572f, 12.6644f, 7.00000f, 18.0572f,
                    14.0000f, 2.00000f, 16.0000f, 14.0000f, 3.31963f, 17.3196f,
                    14.0000f, 5.13375f, 18.0000f, 14.0000f, 7.00000f, 18.0000f, },
            { 14.0000f, 2.00000f, 16.0000f, 14.0000f, 3.31963f, 17.3196f,
                    14.0000f, 5.13375f, 18.0000f, 14.0000f, 7.00000f, 18.0000f,
                    15.3354f, 2.00000f, 16.0000f, 15.3354f, 3.31963f, 17.3196f,
                    15.0055f, 5.13375f, 17.9569f, 15.0055f, 7.00000f, 17.9569f,
                    16.6794f, 2.01251f, 16.1985f, 16.4870f, 3.43424f, 17.3547f,
                    16.0130f, 5.15743f, 17.8973f, 16.0130f, 7.00000f, 17.8973f,
                    18.0000f, 2.00000f, 16.0000f, 17.8076f, 3.42172f, 17.1561f,
                    17.0000f, 5.15743f, 17.7000f, 17.0000f, 7.00000f, 17.7000f, },
            { 18.0000f, 2.00000f, 16.0000f, 17.8076f, 3.42172f, 17.1561f,
                    17.0000f, 5.15743f, 17.7000f, 17.0000f, 7.00000f, 17.7000f,
                    19.0646f, 1.98991f, 15.8399f, 18.8722f, 3.41164f, 16.9960f,
                    17.6994f, 5.15743f, 17.5601f, 17.6994f, 7.00000f, 17.5601f,
                    20.2213f, 1.34332f, 15.3489f, 19.4209f, 2.62530f, 16.5462f,
                    19.1846f, 5.31432f, 16.8753f, 19.0000f, 7.00000f, 17.0000f,
                    21.0000f, 2.00000f, 15.0000f, 20.1996f, 3.28198f, 16.1973f,
                    19.0000f, 7.00000f, 17.0000f, 19.0000f, 7.00000f, 17.0000f, },
            { 21.0000f, 2.00000f, 15.0000f, 20.1996f, 3.28198f, 16.1973f,
                    19.0000f, 7.00000f, 17.0000f, 19.0000f, 7.00000f, 17.0000f,
                    22.2794f, 3.07898f, 14.4267f, 21.4790f, 4.36096f, 15.6240f,
                    20.5517f, 6.99999f, 15.9525f, 19.0000f, 7.00000f, 17.0000f,
                    21.9821f, 5.49730f, 15.2768f, 21.4642f, 7.00000f, 16.0536f,
                    21.4790f, 9.63903f, 15.6240f, 20.1996f, 10.7180f, 16.1973f,
                    22.5000f, 7.00000f, 14.5000f, 21.9821f, 8.50269f, 15.2768f,
                    22.2794f, 10.9210f, 14.4267f, 21.0000f, 12.0000f, 15.0000f, },
            { 21.0000f, 12.0000f, 15.0000f, 20.1996f, 10.7180f, 16.1973f,
                    19.0000f, 7.00000f, 17.0000f, 19.0000f, 7.00000f, 17.0000f,
                    20.2213f, 12.6566f, 15.3489f, 19.4209f, 11.3746f, 16.5462f,
                    19.1846f, 8.68567f, 16.8753f, 19.0000f, 7.00000f, 17.0000f,
                    19.0646f, 12.0100f, 15.8399f, 18.8722f, 10.5883f, 16.9960f,
                    17.6994f, 8.84256f, 17.5601f, 17.6994f, 7.00000f, 17.5601f,
                    18.0000f, 12.0000f, 16.0000f, 17.8076f, 10.5782f, 17.1561f,
                    17.0000f, 8.84256f, 17.7000f, 17.0000f, 7.00000f, 17.7000f, },
            { 18.0000f, 12.0000f, 16.0000f, 17.8076f, 10.5782f, 17.1561f,
                    17.0000f, 8.84256f, 17.7000f, 17.0000f, 7.00000f, 17.7000f,
                    16.6783f, 11.9874f, 16.1987f, 16.4859f, 10.5657f, 17.3549f,
                    16.0130f, 8.84256f, 17.8973f, 16.0130f, 7.00000f, 17.8973f,
                    15.3264f, 12.1643f, 16.0012f, 15.3324f, 10.7567f, 17.2083f,
                    15.0055f, 8.85429f, 17.9569f, 15.0055f, 7.00000f, 17.9569f,
                    14.0000f, 12.0000f, 16.0000f, 14.0060f, 10.5924f, 17.2071f,
                    14.0000f, 8.85429f, 18.0000f, 14.0000f, 7.00000f, 18.0000f, },
            { 14.0000f, 12.0000f, 16.0000f, 14.0060f, 10.5924f, 17.2071f,
                    14.0000f, 8.85429f, 18.0000f, 14.0000f, 7.00000f, 18.0000f,
                    12.6322f, 11.8305f, 15.9987f, 12.6382f, 10.4229f, 17.2058f,
                    12.6644f, 8.85429f, 18.0572f, 12.6644f, 7.00000f, 18.0572f,
                    11.1786f, 11.1102f, 15.8546f, 11.3641f, 10.1538f, 17.0682f,
                    11.3151f, 8.57346f, 17.6712f, 11.2968f, 7.00000f, 17.6757f,
                    10.0000f, 11.0000f, 16.0000f, 10.0023f, 9.92533f, 17.3646f,
                    10.0182f, 8.57346f, 17.9954f, 10.0000f, 7.00000f, 18.0000f, },
            { 10.0000f, 7.00000f, 18.0000f, 9.97797f, 5.10529f, 18.0055f,
                    10.0036f, 3.39890f, 17.7763f, 10.0000f, 2.00000f, 16.0000f,
                    8.54835f, 7.00000f, 18.3629f, 8.52633f, 5.10529f, 18.3684f,
                    8.65977f, 3.62422f, 18.0685f, 7.86846f, 2.71708f, 16.9202f,
                    7.49631f, 7.00000f, 20.0000f, 7.49631f, 5.45253f, 20.0000f,
                    7.26780f, 3.46826f, 18.6874f, 6.00000f, 3.00000f, 18.0000f,
                    6.00000f, 7.00000f, 20.0000f, 6.00000f, 5.45253f, 20.0000f,
                    6.00000f, 3.00000f, 18.0000f, 6.00000f, 3.00000f, 18.0000f, },
            { 6.00000f, 7.00000f, 20.0000f, 6.00000f, 5.45253f, 20.0000f,
                    6.00000f, 3.00000f, 18.0000f, 6.00000f, 3.00000f, 18.0000f,
                    4.70898f, 7.00000f, 20.0000f, 4.70898f, 5.45253f, 20.0000f,
                    5.43536f, 3.74871f, 18.6055f, 6.00000f, 3.00000f, 18.0000f,
                    3.62697f, 7.00000f, 19.1285f, 3.62697f, 5.28417f, 19.1285f,
                    4.35824f, 3.90451f, 17.7575f, 5.28514f, 2.90705f, 16.7135f,
                    3.00000f, 7.00000f, 18.0000f, 2.99999f, 5.28417f, 18.0000f,
                    4.07310f, 3.99745f, 17.0440f, 5.00000f, 3.00000f, 16.0000f, },
            { 3.00000f, 7.00000f, 18.0000f, 2.99999f, 5.28417f, 18.0000f,
                    4.07310f, 3.99745f, 17.0440f, 5.00000f, 3.00000f, 16.0000f,
                    1.79545f, 7.00000f, 15.8318f, 1.79545f, 5.28417f, 15.8318f,
                    3.32781f, 4.24038f, 15.1790f, 4.25471f, 3.24292f, 14.1350f,
                    0.794020f, 7.00000f, 13.4717f, 0.794020f, 5.60875f,
                    13.4717f, 1.51717f, 3.42451f, 12.7792f, 2.88595f, 3.40717f,
                    12.5308f, 1.00000f, 7.00000f, 11.0000f, 1.00000f, 5.60875f,
                    11.0000f, 1.63122f, 4.01734f, 10.8484f, 3.00000f, 4.00000f,
                    10.6000f, },
            { 1.00000f, 7.00000f, 11.0000f, 1.00000f, 5.60875f, 11.0000f,
                    1.63122f, 4.01734f, 10.8484f, 3.00000f, 4.00000f, 10.6000f,
                    1.16542f, 7.00000f, 9.01491f, 1.16542f, 5.60875f, 9.01491f,
                    1.71610f, 4.45857f, 9.41132f, 3.08488f, 4.44123f, 9.16289f,
                    2.70364f, 7.00000f, 7.51241f, 2.70364f, 6.27240f, 7.51241f,
                    3.60899f, 5.37796f, 7.63476f, 4.12348f, 5.37796f, 8.14925f,
                    4.00000f, 7.00000f, 6.00000f, 4.00000f, 6.27240f, 6.00000f,
                    4.48551f, 5.80000f, 6.48551f, 5.00000f, 5.80000f, 7.00000f, },
            { 4.00000f, 7.00000f, 6.00000f, 4.00000f, 6.27240f, 6.00000f,
                    4.48551f, 5.80000f, 6.48551f, 5.00000f, 5.80000f, 7.00000f,
                    4.79829f, 7.00000f, 5.06865f, 4.79829f, 6.27240f, 5.06865f,
                    5.03793f, 6.06598f, 5.76119f, 5.55242f, 6.06598f, 6.27568f,
                    5.82508f, 7.00000f, 4.35247f, 5.82508f, 6.44606f, 4.35247f,
                    6.08632f, 5.95431f, 4.69838f, 6.08632f, 5.95431f, 5.25232f,
                    7.00000f, 7.00000f, 4.00000f, 7.00000f, 6.44606f, 4.00000f,
                    7.00000f, 6.00000f, 4.44606f, 7.00000f, 6.00000f, 5.00000f, },
            { 7.00000f, 7.00000f, 4.00000f, 7.00000f, 6.44606f, 4.00000f,
                    7.00000f, 6.00000f, 4.44606f, 7.00000f, 6.00000f, 5.00000f,
                    7.69551f, 7.00000f, 3.79134f, 7.69551f, 6.44606f, 3.79134f,
                    7.66874f, 6.03343f, 4.26137f, 7.66874f, 6.03343f, 4.81531f,
                    9.00000f, 7.00000f, 4.50000f, 8.63511f, 6.54994f, 4.22633f,
                    8.59332f, 5.79666f, 4.83390f, 9.00000f, 6.00000f, 5.50000f,
                    9.00000f, 7.00000f, 4.50000f, 9.00000f, 7.00000f, 4.50000f,
                    9.00000f, 6.00000f, 5.50000f, 9.00000f, 6.00000f, 5.50000f, },
            { 9.00000f, 7.00000f, 4.50000f, 9.00000f, 7.00000f, 4.50000f,
                    9.00000f, 6.00000f, 5.50000f, 9.00000f, 6.00000f, 5.50000f,
                    9.00000f, 7.00000f, 4.50000f, 9.43205f, 7.00000f, 4.82404f,
                    9.44642f, 6.22321f, 5.33934f, 9.00000f, 6.00000f, 5.50000f,
                    9.00000f, 8.00000f, 5.50000f, 9.44642f, 7.77679f, 5.33934f,
                    9.09980f, 7.00000f, 5.97504f, 9.00000f, 7.00000f, 6.00000f,
                    9.00000f, 8.00000f, 5.50000f, 9.00000f, 8.00000f, 5.50000f,
                    9.00000f, 7.00000f, 6.00000f, 9.00000f, 7.00000f, 6.00000f, },
            { 10.0000f, 2.00000f, 8.00000f, 8.93204f, 3.06795f, 8.00000f,
                    8.96952f, 4.43964f, 7.67212f, 10.0000f, 6.00000f, 8.00000f,
                    9.87124f, 1.95609f, 9.38125f, 8.80329f, 3.02405f, 9.38125f,
                    8.74608f, 4.46130f, 8.42383f, 9.56588f, 5.70264f, 8.68468f,
                    9.34408f, 1.50592f, 9.94049f, 8.51649f, 2.09617f, 9.55423f,
                    9.59858f, 3.81408f, 9.77086f, 9.59783f, 5.18668f, 9.40552f,
                    9.00000f, 1.50000f, 12.0000f, 7.73438f, 2.55240f, 10.0865f,
                    9.00075f, 3.62739f, 10.3653f, 9.00000f, 5.00000f, 10.0000f, },
            { 9.00000f, 1.50000f, 12.0000f, 7.73438f, 2.55240f, 10.0865f,
                    9.00075f, 3.62739f, 10.3653f, 9.00000f, 5.00000f, 10.0000f,
                    9.00000f, 1.50000f, 12.0000f, 7.15956f, 2.88771f, 10.4777f,
                    8.48141f, 3.46521f, 10.8817f, 8.48065f, 4.83782f, 10.5164f,
                    9.00000f, 1.50000f, 12.0000f, 7.15956f, 2.88771f, 10.4777f,
                    8.32175f, 3.72942f, 10.9234f, 7.85973f, 4.98761f, 10.5389f,
                    9.00000f, 1.50000f, 12.0000f, 7.45296f, 2.59077f, 11.3891f,
                    7.91171f, 3.25991f, 11.3835f, 7.00000f, 5.00000f, 11.0000f, },
            { 9.00000f, 1.50000f, 12.0000f, 7.45296f, 2.59077f, 11.3891f,
                    7.91171f, 3.25991f, 11.3835f, 7.00000f, 5.00000f, 11.0000f,
                    9.07887f, 1.31725f, 13.1541f, 7.97031f, 2.06717f, 12.9963f,
                    7.41411f, 2.69014f, 11.9419f, 6.75115f, 3.94166f, 11.6699f,
                    9.52420f, 0.976845f, 14.7029f, 8.32713f, 0.844520f,
                    14.5403f, 8.27262f, 2.45611f, 13.7536f, 7.52117f, 3.19943f,
                    12.9181f, 10.0000f, 2.00000f, 16.0000f, 7.84991f, 1.51717f,
                    15.3966f, 7.75144f, 2.05668f, 14.8355f, 7.00000f, 2.80000f,
                    14.0000f, },
            { 10.0000f, 2.00000f, 16.0000f, 7.84991f, 1.51717f, 15.3966f,
                    7.75144f, 2.05668f, 14.8355f, 7.00000f, 2.80000f, 14.0000f,
                    7.86846f, 2.71708f, 16.9202f, 7.38876f, 2.16717f, 16.2241f,
                    7.34956f, 1.74867f, 15.6698f, 6.59812f, 2.49199f, 14.8342f,
                    6.00000f, 3.00000f, 18.0000f, 6.83143f, 1.96557f, 17.1732f,
                    5.81231f, 2.33975f, 16.1197f, 5.52717f, 2.43269f, 15.4062f,
                    6.00000f, 3.00000f, 18.0000f, 6.00000f, 3.00000f, 18.0000f,
                    5.28514f, 2.90705f, 16.7135f, 5.00000f, 3.00000f, 16.0000f, },
            { 5.00000f, 3.00000f, 16.0000f, 5.52717f, 2.43269f, 15.4062f,
                    6.59812f, 2.49199f, 14.8342f, 7.00000f, 2.80000f, 14.0000f,
                    4.25471f, 3.24292f, 14.1350f, 4.78188f, 2.67562f, 13.5412f,
                    5.34262f, 3.73390f, 13.4382f, 5.74450f, 4.04191f, 12.6039f,
                    2.88595f, 3.40717f, 12.5308f, 4.34454f, 3.38869f, 12.2661f,
                    6.08167f, 3.98531f, 10.4018f, 7.19502f, 4.53322f, 11.1711f,
                    3.00000f, 4.00000f, 10.6000f, 4.45858f, 3.98151f, 10.3352f,
                    4.69919f, 5.29811f, 9.44672f, 5.50000f, 7.00000f, 10.0000f, },
            { 3.00000f, 4.00000f, 10.6000f, 4.45858f, 3.98151f, 10.3352f,
                    4.69919f, 5.29811f, 9.44672f, 5.50000f, 7.00000f, 10.0000f,
                    3.08488f, 4.44123f, 9.16289f, 4.54347f, 4.42275f, 8.89816f,
                    4.26221f, 5.71307f, 9.14480f, 4.26221f, 7.00000f, 9.14480f,
                    4.12348f, 5.37796f, 8.14925f, 4.62387f, 5.37796f, 8.64963f,
                    5.73017f, 6.29235f, 8.71953f, 5.73017f, 7.00000f, 8.71953f,
                    5.00000f, 5.80000f, 7.00000f, 5.50038f, 5.80000f, 7.50038f,
                    6.00000f, 6.29235f, 8.00000f, 6.00000f, 7.00000f, 8.00000f, },
            { 5.00000f, 5.80000f, 7.00000f, 5.50038f, 5.80000f, 7.50038f,
                    6.00000f, 6.29235f, 8.00000f, 6.00000f, 7.00000f, 8.00000f,
                    5.55242f, 6.06598f, 6.27568f, 6.05280f, 6.06598f, 6.77606f,
                    6.26818f, 6.29235f, 7.28483f, 6.26818f, 7.00000f, 7.28483f,
                    6.08632f, 5.95431f, 5.25232f, 6.08632f, 5.95431f, 5.80147f,
                    6.36448f, 6.45084f, 6.42367f, 6.36448f, 7.00000f, 6.42367f,
                    7.00000f, 6.00000f, 5.00000f, 7.00000f, 6.00000f, 5.54915f,
                    7.00000f, 6.45084f, 6.00000f, 7.00000f, 7.00000f, 6.00000f, },
            { 7.00000f, 6.00000f, 5.00000f, 7.00000f, 6.00000f, 5.54915f,
                    7.00000f, 6.45084f, 6.00000f, 7.00000f, 7.00000f, 6.00000f,
                    7.66874f, 6.03343f, 4.81531f, 7.66874f, 6.03343f, 5.36447f,
                    7.55880f, 6.45084f, 5.62746f, 7.55880f, 7.00000f, 5.62746f,
                    9.00000f, 6.00000f, 5.50000f, 8.65645f, 5.82822f, 5.63149f,
                    8.39835f, 6.57583f, 6.15041f, 9.00000f, 7.00000f, 6.00000f,
                    9.00000f, 6.00000f, 5.50000f, 9.00000f, 6.00000f, 5.50000f,
                    9.00000f, 7.00000f, 6.00000f, 9.00000f, 7.00000f, 6.00000f, },
            { 10.0000f, 11.0000f, 16.0000f, 10.0023f, 9.92533f, 17.3646f,
                    10.0182f, 8.57346f, 17.9954f, 10.0000f, 7.00000f, 18.0000f,
                    7.93998f, 10.5043f, 16.6387f, 8.61321f, 9.69225f, 17.6669f,
                    8.56665f, 8.57346f, 18.3583f, 8.54835f, 7.00000f, 18.3629f,
                    6.00000f, 11.0000f, 18.0000f, 7.40668f, 10.2558f, 18.7441f,
                    7.49631f, 8.54969f, 20.0000f, 7.49631f, 7.00000f, 20.0000f,
                    6.00000f, 11.0000f, 18.0000f, 6.00000f, 11.0000f, 18.0000f,
                    6.00000f, 8.54969f, 20.0000f, 6.00000f, 7.00000f, 20.0000f, },
            { 6.00000f, 11.0000f, 18.0000f, 6.00000f, 11.0000f, 18.0000f,
                    6.00000f, 8.54969f, 20.0000f, 6.00000f, 7.00000f, 20.0000f,
                    6.00000f, 11.0000f, 18.0000f, 5.54861f, 10.3555f, 18.6444f,
                    4.70898f, 8.54969f, 20.0000f, 4.70898f, 7.00000f, 20.0000f,
                    5.28802f, 11.0938f, 16.7207f, 4.36112f, 10.0964f, 17.7647f,
                    3.62697f, 8.71582f, 19.1285f, 3.62697f, 7.00000f, 19.1285f,
                    5.00000f, 11.0000f, 16.0000f, 4.07310f, 10.0025f, 17.0440f,
                    3.00000f, 8.71582f, 18.0000f, 3.00000f, 7.00000f, 18.0000f, },
            { 5.00000f, 11.0000f, 16.0000f, 4.07310f, 10.0025f, 17.0440f,
                    3.00000f, 8.71582f, 18.0000f, 3.00000f, 7.00000f, 18.0000f,
                    4.25471f, 10.7570f, 14.1350f, 3.32781f, 9.75961f, 15.1790f,
                    1.79545f, 8.71582f, 15.8318f, 1.79545f, 7.00000f, 15.8318f,
                    2.88595f, 10.5928f, 12.5308f, 1.51717f, 10.5754f, 12.7792f,
                    0.794020f, 8.39124f, 13.4717f, 0.794020f, 7.00000f,
                    13.4717f, 3.00000f, 10.0000f, 10.6000f, 1.63122f, 9.98265f,
                    10.8484f, 1.00000f, 8.39124f, 11.0000f, 1.00000f, 7.00000f,
                    11.0000f, },
            { 3.00000f, 10.0000f, 10.6000f, 1.63122f, 9.98265f, 10.8484f,
                    1.00000f, 8.39124f, 11.0000f, 1.00000f, 7.00000f, 11.0000f,
                    3.08488f, 9.55876f, 9.16289f, 1.71610f, 9.54142f, 9.41132f,
                    1.16542f, 8.39124f, 9.01491f, 1.16542f, 7.00000f, 9.01491f,
                    4.12348f, 8.62203f, 8.14925f, 3.60899f, 8.62203f, 7.63476f,
                    2.70364f, 7.72759f, 7.51241f, 2.70364f, 7.00000f, 7.51241f,
                    5.00000f, 8.20000f, 7.00000f, 4.48551f, 8.20000f, 6.48550f,
                    4.00000f, 7.72759f, 6.00000f, 4.00000f, 7.00000f, 6.00000f, },
            { 5.00000f, 8.20000f, 7.00000f, 4.48551f, 8.20000f, 6.48550f,
                    4.00000f, 7.72759f, 6.00000f, 4.00000f, 7.00000f, 6.00000f,
                    5.55241f, 7.93401f, 6.27568f, 5.03792f, 7.93401f, 5.76119f,
                    4.79829f, 7.72759f, 5.06865f, 4.79829f, 7.00000f, 5.06865f,
                    6.08632f, 8.04568f, 5.25232f, 6.08632f, 8.04568f, 4.69838f,
                    5.82508f, 7.55393f, 4.35247f, 5.82508f, 7.00000f, 4.35247f,
                    7.00000f, 8.00000f, 5.00000f, 7.00000f, 8.00000f, 4.44606f,
                    7.00000f, 7.55393f, 4.00000f, 7.00000f, 7.00000f, 4.00000f, },
            { 7.00000f, 8.00000f, 5.00000f, 7.00000f, 8.00000f, 4.44606f,
                    7.00000f, 7.55393f, 4.00000f, 7.00000f, 7.00000f, 4.00000f,
                    7.66874f, 7.96656f, 4.81531f, 7.66874f, 7.96656f, 4.26138f,
                    7.69551f, 7.55393f, 3.79134f, 7.69551f, 7.00000f, 3.79134f,
                    9.00000f, 8.00000f, 5.50000f, 8.59332f, 8.20333f, 4.83390f,
                    8.63511f, 7.45005f, 4.22633f, 9.00000f, 7.00000f, 4.50000f,
                    9.00000f, 8.00000f, 5.50000f, 9.00000f, 8.00000f, 5.50000f,
                    9.00000f, 7.00000f, 4.50000f, 9.00000f, 7.00000f, 4.50000f, },
            { 10.0000f, 8.00000f, 8.00000f, 8.96952f, 9.56035f, 7.67212f,
                    8.93204f, 10.9320f, 8.00000f, 10.0000f, 12.0000f, 8.00000f,
                    9.56588f, 8.29735f, 8.68468f, 8.74608f, 9.53870f, 8.42383f,
                    8.80346f, 10.9758f, 9.37941f, 9.87141f, 12.0438f, 9.37941f,
                    9.59783f, 8.81331f, 9.40552f, 9.59859f, 10.1923f, 9.77256f,
                    8.27272f, 11.9399f, 9.68199f, 9.15043f, 12.6614f, 10.0622f,
                    9.00000f, 9.00000f, 10.0000f, 9.00075f, 10.3789f, 10.3670f,
                    7.51132f, 11.2958f, 10.3210f, 9.00000f, 12.5000f, 12.0000f, },
            { 9.00000f, 9.00000f, 10.0000f, 9.00075f, 10.3789f, 10.3670f,
                    7.51132f, 11.2958f, 10.3210f, 9.00000f, 12.5000f, 12.0000f,
                    9.00000f, 9.00000f, 10.0000f, 8.48141f, 10.5411f, 10.8834f,
                    7.07052f, 10.9228f, 10.6910f, 7.30706f, 11.1052f, 11.4309f,
                    9.00000f, 9.00000f, 10.0000f, 8.48141f, 10.5411f, 10.8834f,
                    8.32883f, 10.2840f, 10.9264f, 7.91955f, 10.7550f, 11.3868f,
                    9.00000f, 9.00000f, 10.0000f, 8.48065f, 9.16217f, 10.5164f,
                    7.86080f, 9.01240f, 10.5383f, 7.00000f, 9.00000f, 11.0000f, },
            { 7.00000f, 9.00000f, 11.0000f, 7.91955f, 10.7550f, 11.3868f,
                    7.30706f, 11.1052f, 11.4309f, 9.00000f, 12.5000f, 12.0000f,
                    6.74950f, 10.0653f, 11.6744f, 7.41955f, 11.3303f, 11.9493f,
                    7.86345f, 11.5343f, 13.1713f, 8.96405f, 12.4448f, 13.3410f,
                    7.52546f, 10.7873f, 12.9152f, 8.32087f, 11.3878f, 13.7595f,
                    8.49662f, 12.3274f, 14.3212f, 9.51945f, 12.2157f, 14.4584f,
                    7.00000f, 11.2000f, 14.0000f, 7.79541f, 11.8004f, 14.8442f,
                    7.99336f, 11.7065f, 15.1118f, 10.0000f, 11.0000f, 16.0000f, },
            { 7.00000f, 11.2000f, 14.0000f, 7.79541f, 11.8004f, 14.8442f,
                    7.99336f, 11.7065f, 15.1118f, 10.0000f, 11.0000f, 16.0000f,
                    6.59576f, 11.5174f, 14.8344f, 7.39118f, 12.1179f, 15.6787f,
                    7.47421f, 11.0661f, 15.9274f, 7.93998f, 10.5043f, 16.6387f,
                    5.52943f, 11.5697f, 15.4036f, 5.81745f, 11.6636f, 16.1244f,
                    6.95530f, 11.8030f, 17.1969f, 6.00000f, 11.0000f, 18.0000f,
                    5.00000f, 11.0000f, 16.0000f, 5.28802f, 11.0938f, 16.7207f,
                    6.00000f, 11.0000f, 18.0000f, 6.00000f, 11.0000f, 18.0000f, },
            { 7.00000f, 11.2000f, 14.0000f, 6.59576f, 11.5174f, 14.8344f,
                    5.52943f, 11.5697f, 15.4036f, 5.00000f, 11.0000f, 16.0000f,
                    5.63935f, 10.1728f, 12.5557f, 5.23511f, 10.4902f, 13.3902f,
                    4.78415f, 11.3268f, 13.5386f, 4.25471f, 10.7570f, 14.1350f,
                    7.18618f, 9.45391f, 11.1650f, 6.07284f, 10.0018f, 10.3957f,
                    4.34454f, 10.6113f, 12.2661f, 2.88595f, 10.5928f, 12.5308f,
                    5.50000f, 7.00000f, 10.0000f, 4.69955f, 8.70113f, 9.44696f,
                    4.45858f, 10.0184f, 10.3352f, 3.00000f, 10.0000f, 10.6000f, },
            { 5.50000f, 7.00000f, 10.0000f, 4.69955f, 8.70113f, 9.44696f,
                    4.45858f, 10.0184f, 10.3352f, 3.00000f, 10.0000f, 10.6000f,
                    4.26221f, 7.00000f, 9.14480f, 4.26221f, 8.28692f, 9.14480f,
                    4.54346f, 9.57725f, 8.89816f, 3.08488f, 9.55876f, 9.16289f,
                    5.73017f, 7.00000f, 8.71953f, 5.73017f, 7.70764f, 8.71953f,
                    4.62387f, 8.62203f, 8.64963f, 4.12348f, 8.62203f, 8.14925f,
                    6.00000f, 7.00000f, 8.00000f, 6.00000f, 7.70764f, 8.00000f,
                    5.50038f, 8.20000f, 7.50038f, 5.00000f, 8.20000f, 7.00000f, },
            { 6.00000f, 7.00000f, 8.00000f, 6.00000f, 7.70764f, 8.00000f,
                    5.50038f, 8.20000f, 7.50038f, 5.00000f, 8.20000f, 7.00000f,
                    6.26818f, 7.00000f, 7.28483f, 6.26818f, 7.70764f, 7.28483f,
                    6.05280f, 7.93401f, 6.77606f, 5.55241f, 7.93401f, 6.27568f,
                    6.36448f, 7.00000f, 6.42367f, 6.36448f, 7.54915f, 6.42367f,
                    6.08632f, 8.04568f, 5.80147f, 6.08632f, 8.04568f, 5.25232f,
                    7.00000f, 7.00000f, 6.00000f, 7.00000f, 7.54915f, 6.00000f,
                    7.00000f, 8.00000f, 5.54915f, 7.00000f, 8.00000f, 5.00000f, },
            { 7.00000f, 7.00000f, 6.00000f, 7.00000f, 7.54915f, 6.00000f,
                    7.00000f, 8.00000f, 5.54915f, 7.00000f, 8.00000f, 5.00000f,
                    7.55880f, 7.00000f, 5.62746f, 7.55880f, 7.54915f, 5.62746f,
                    7.66874f, 7.96656f, 5.36447f, 7.66874f, 7.96656f, 4.81531f,
                    9.00000f, 7.00000f, 6.00000f, 8.39835f, 7.42416f, 6.15041f,
                    8.65644f, 8.17177f, 5.63149f, 9.00000f, 8.00000f, 5.50000f,
                    9.00000f, 7.00000f, 6.00000f, 9.00000f, 7.00000f, 6.00000f,
                    9.00000f, 8.00000f, 5.50000f, 9.00000f, 8.00000f, 5.50000f, },
            { 10.0000f, 6.00000f, 8.00000f, 10.6753f, 7.02258f, 8.21487f,
                    10.6753f, 6.97741f, 8.21487f, 10.0000f, 8.00000f, 8.00000f,
                    9.56588f, 5.70264f, 8.68468f, 9.96312f, 6.30414f, 8.81107f,
                    9.96312f, 7.69585f, 8.81107f, 9.56588f, 8.29735f, 8.68468f,
                    9.59783f, 5.18668f, 9.40552f, 9.59712f, 6.48744f, 9.05930f,
                    9.59712f, 7.51255f, 9.05930f, 9.59783f, 8.81331f, 9.40552f,
                    9.00000f, 5.00000f, 10.0000f, 8.99928f, 6.30075f, 9.65378f,
                    8.99928f, 7.69924f, 9.65378f, 9.00000f, 9.00000f, 10.0000f, },
            { 9.00000f, 5.00000f, 10.0000f, 8.99928f, 6.30075f, 9.65378f,
                    8.99928f, 7.69924f, 9.65378f, 9.00000f, 9.00000f, 10.0000f,
                    8.48065f, 4.83782f, 10.5164f, 8.47994f, 6.13857f, 10.1702f,
                    8.47994f, 7.86142f, 10.1702f, 8.48065f, 9.16217f, 10.5164f,
                    7.85973f, 4.98761f, 10.5389f, 7.39107f, 6.26389f, 10.1488f,
                    7.39108f, 7.73610f, 10.1488f, 7.86080f, 9.01240f, 10.5383f,
                    7.00000f, 5.00000f, 11.0000f, 6.60445f, 6.86590f, 10.2723f,
                    6.60445f, 7.13409f, 10.2723f, 7.00000f, 9.00000f, 11.0000f, },
            { 5.50000f, 7.00000f, 10.0000f, 7.17220f, 4.74482f, 11.1553f,
                    6.28350f, 6.33090f, 10.7170f, 7.00000f, 5.00000f, 11.0000f,
                    5.50000f, 7.00000f, 10.0000f, 6.57446f, 7.00000f, 10.7423f,
                    6.27560f, 7.11757f, 10.3239f, 6.60445f, 6.86590f, 10.2723f,
                    5.50000f, 7.00000f, 10.0000f, 6.57446f, 7.00000f, 10.7423f,
                    6.27560f, 6.88242f, 10.3238f, 6.60445f, 7.13409f, 10.2723f,
                    5.50000f, 7.00000f, 10.0000f, 7.16443f, 9.24469f, 11.1499f,
                    6.28292f, 7.66800f, 10.7168f, 7.00000f, 9.00000f, 11.0000f, },
            { 7.00000f, 2.80000f, 14.0000f, 7.52117f, 3.19943f, 12.9181f,
                    6.75115f, 3.94166f, 11.6699f, 7.00000f, 5.00000f, 11.0000f,
                    7.00000f, 2.80000f, 14.0000f, 6.26568f, 4.44134f, 11.5221f,
                    6.29894f, 4.79533f, 11.4844f, 6.28350f, 6.33090f, 10.7170f,
                    7.00000f, 2.80000f, 14.0000f, 6.26568f, 4.44134f, 11.5221f,
                    7.22445f, 4.54771f, 11.1914f, 7.17220f, 4.74482f, 11.1553f,
                    7.00000f, 2.80000f, 14.0000f, 5.74450f, 4.04191f, 12.6039f,
                    7.19502f, 4.53322f, 11.1711f, 5.50000f, 7.00000f, 10.0000f, },
            { 7.00000f, 11.2000f, 14.0000f, 5.63935f, 10.1728f, 12.5557f,
                    7.18618f, 9.45391f, 11.1650f, 5.50000f, 7.00000f, 10.0000f,
                    7.00000f, 11.2000f, 14.0000f, 6.16481f, 9.76016f, 11.4710f,
                    7.21561f, 9.43943f, 11.1853f, 7.16443f, 9.24469f, 11.1499f,
                    7.00000f, 11.2000f, 14.0000f, 6.16481f, 9.76016f, 11.4710f,
                    6.29730f, 9.21164f, 11.4888f, 6.28292f, 7.66800f, 10.7168f,
                    7.00000f, 11.2000f, 14.0000f, 7.52546f, 10.7873f, 12.9152f,
                    6.74950f, 10.0653f, 11.6744f, 7.00000f, 9.00000f, 11.0000f, },
            { 6.00000f, 4.00000f, 18.0000f, 6.17178f, 3.61615f, 18.9501f,
                    7.00000f, 2.00000f, 20.0000f, 7.00000f, 2.00000f, 20.0000f,
                    5.82238f, 3.78567f, 15.7951f, 5.99416f, 3.40183f, 16.7453f,
                    6.36326f, 1.62520f, 19.4440f, 7.00000f, 2.00000f, 20.0000f,
                    6.99191f, -0.0338653f, 16.2383f, 7.06775f, -0.0553456f,
                    17.3163f, 6.99797f, -0.708357f, 18.6050f, 7.50000f,
                    -1.00000f, 19.0000f, 8.00000f, -2.00000f, 16.0000f,
                    8.07584f, -2.02148f, 17.0779f, 7.50000f, -1.00000f,
                    19.0000f, 7.50000f, -1.00000f, 19.0000f, },
            { 6.00000f, 4.00000f, 18.0000f, 5.82238f, 3.78567f, 15.7951f,
                    6.99191f, -0.0338653f, 16.2383f, 8.00000f, -2.00000f,
                    16.0000f, 5.77637f, 4.49967f, 16.7631f, 5.59875f, 4.28535f,
                    14.5582f, 6.86100f, 0.00320839f, 14.3779f, 7.86909f,
                    -1.96292f, 14.1395f, 5.91173f, 4.38739f, 15.2929f,
                    5.55206f, 2.71906f, 15.1499f, 6.48478f, 0.0405683f,
                    12.5429f, 7.05435f, -1.47828f, 11.9935f, 6.00000f,
                    4.00000f, 14.0000f, 5.64033f, 2.33167f, 13.8569f, 5.93043f,
                    1.51884f, 11.5493f, 6.50000f, 0.00000f, 11.0000f, },
            { 6.00000f, 4.00000f, 14.0000f, 5.64033f, 2.33167f, 13.8569f,
                    5.93043f, 1.51884f, 11.5493f, 6.50000f, 0.00000f, 11.0000f,
                    6.00000f, 4.00000f, 14.0000f, 5.72105f, 1.97742f, 12.6746f,
                    5.71956f, 2.08115f, 11.1714f, 6.28913f, 0.562308f,
                    10.6220f, 6.00000f, 4.00000f, 14.0000f, 5.72105f, 1.97742f,
                    12.6746f, 5.95300f, 2.11991f, 11.7283f, 6.50000f, 2.00000f,
                    11.0000f, 6.00000f, 4.00000f, 14.0000f, 6.08071f, 3.64575f,
                    12.8176f, 6.50000f, 2.00000f, 11.0000f, 6.50000f, 2.00000f,
                    11.0000f, },
            { 7.00000f, 4.00000f, 18.0000f, 8.01538f, 2.03531f, 17.7856f,
                    9.25405f, -1.57894f, 18.1668f, 9.00000f, -2.00000f,
                    16.0000f, 7.27289f, 3.61974f, 18.9768f, 8.28828f, 1.65505f,
                    18.7624f, 9.43541f, -1.59908f, 19.2725f, 9.18135f,
                    -2.02014f, 17.1057f, 8.50000f, 2.00000f, 20.0000f,
                    8.88419f, 1.96998f, 19.2616f, 9.44404f, -0.614553f,
                    18.6694f, 9.00000f, -1.00000f, 19.0000f, 8.50000f,
                    2.00000f, 20.0000f, 8.50000f, 2.00000f, 20.0000f, 9.00000f,
                    -1.00000f, 19.0000f, 9.00000f, -1.00000f, 19.0000f, },
            { 9.00000f, -2.00000f, 16.0000f, 9.25405f, -1.57894f, 18.1668f,
                    8.01538f, 2.03531f, 17.7856f, 7.00000f, 4.00000f, 18.0000f,
                    8.69788f, -1.96644f, 14.1579f, 8.95194f, -1.54538f,
                    16.3247f, 7.67925f, 2.50367f, 16.5825f, 6.66387f, 4.46835f,
                    16.7968f, 7.84657f, -1.53663f, 12.0021f, 8.19383f,
                    -0.932816f, 13.5421f, 7.49810f, 2.87203f, 14.8067f,
                    6.90135f, 4.38271f, 15.2742f, 7.50000f, 0.00000f, 11.0000f,
                    7.84726f, 0.603817f, 12.5400f, 7.59675f, 2.48932f,
                    13.5324f, 7.00000f, 4.00000f, 14.0000f, },
            { 7.50000f, 0.00000f, 11.0000f, 7.84726f, 0.603817f, 12.5400f,
                    7.59675f, 2.48932f, 13.5324f, 7.00000f, 4.00000f, 14.0000f,
                    7.50000f, 0.00000f, 11.0000f, 7.72215f, 1.15853f, 12.1782f,
                    7.68730f, 2.13801f, 12.3627f, 7.09055f, 3.64868f, 12.8302f,
                    7.50000f, 0.00000f, 11.0000f, 7.72215f, 1.15853f, 12.1782f,
                    7.80364f, 2.19032f, 11.7976f, 7.50000f, 2.00000f, 11.0000f,
                    7.50000f, 0.00000f, 11.0000f, 7.37488f, 0.554718f,
                    10.6382f, 7.50000f, 2.00000f, 11.0000f, 7.50000f, 2.00000f,
                    11.0000f, },
            { 6.00000f, 4.00000f, 18.0000f, 6.04662f, 4.05625f, 18.5787f,
                    6.73343f, 4.51577f, 18.0562f, 7.00000f, 4.00000f, 18.0000f,
                    6.17178f, 3.61615f, 18.9501f, 6.21840f, 3.67240f, 19.5288f,
                    7.00633f, 4.13552f, 19.0331f, 7.27289f, 3.61974f, 18.9768f,
                    7.00000f, 2.00000f, 20.0000f, 6.87917f, 2.69654f, 19.9349f,
                    7.96063f, 3.27172f, 19.8070f, 8.50000f, 2.00000f, 20.0000f,
                    7.00000f, 2.00000f, 20.0000f, 7.00000f, 2.00000f, 20.0000f,
                    8.50000f, 2.00000f, 20.0000f, 8.50000f, 2.00000f, 20.0000f, },
            { 7.00000f, 2.00000f, 20.0000f, 7.00000f, 2.00000f, 20.0000f,
                    8.50000f, 2.00000f, 20.0000f, 8.50000f, 2.00000f, 20.0000f,
                    7.00000f, 2.00000f, 20.0000f, 7.42866f, 0.886911f,
                    20.3024f, 8.71101f, 1.41080f, 20.1671f, 8.50000f, 2.00000f,
                    20.0000f, 7.50000f, -1.00000f, 19.0000f, 7.50022f,
                    -0.462675f, 19.7526f, 8.73395f, -0.336419f, 19.9299f,
                    9.00000f, -1.00000f, 19.0000f, 7.50000f, -1.00000f,
                    19.0000f, 7.50000f, -1.00000f, 19.0000f, 9.00000f,
                    -1.00000f, 19.0000f, 9.00000f, -1.00000f, 19.0000f, },
            { 7.50000f, -1.00000f, 19.0000f, 7.50000f, -1.00000f, 19.0000f,
                    9.00000f, -1.00000f, 19.0000f, 9.00000f, -1.00000f,
                    19.0000f, 7.50000f, -1.00000f, 19.0000f, 7.86286f,
                    -1.90036f, 18.3200f, 8.92549f, -1.83744f, 18.4231f,
                    9.00000f, -1.00000f, 19.0000f, 8.07584f, -2.02148f,
                    17.0779f, 8.34116f, -2.53894f, 17.0151f, 9.11447f,
                    -2.13097f, 16.5353f, 9.18135f, -2.02014f, 17.1057f,
                    8.00000f, -2.00000f, 16.0000f, 8.26531f, -2.51746f,
                    15.9372f, 8.93312f, -2.11083f, 15.4296f, 9.00000f,
                    -2.00000f, 16.0000f, },
            { 8.00000f, -2.00000f, 16.0000f, 8.26531f, -2.51746f, 15.9372f,
                    8.93312f, -2.11083f, 15.4296f, 9.00000f, -2.00000f,
                    16.0000f, 7.86909f, -1.96292f, 14.1395f, 8.13441f,
                    -2.48038f, 14.0768f, 8.63100f, -2.07728f, 13.5875f,
                    8.69788f, -1.96644f, 14.1579f, 7.05435f, -1.47828f,
                    11.9935f, 7.24070f, -1.97520f, 11.8138f, 7.73145f,
                    -1.73680f, 11.4916f, 7.84657f, -1.53663f, 12.0021f,
                    6.50000f, 0.00000f, 11.0000f, 6.68634f, -0.496919f,
                    10.8202f, 7.38487f, -0.200171f, 10.4894f, 7.50000f,
                    0.00000f, 11.0000f, },
            { 6.50000f, 0.00000f, 11.0000f, 6.68634f, -0.496919f, 10.8202f,
                    7.38487f, -0.200171f, 10.4894f, 7.50000f, 0.00000f,
                    11.0000f, 6.28913f, 0.562308f, 10.6220f, 6.47547f,
                    0.0653883f, 10.4423f, 7.25976f, 0.354546f, 10.1277f,
                    7.37488f, 0.554718f, 10.6382f, 6.50000f, 2.00000f,
                    11.0000f, 6.58882f, 1.48812f, 10.5862f, 7.51059f, 1.62025f,
                    10.6414f, 7.50000f, 2.00000f, 11.0000f, 6.50000f, 2.00000f,
                    11.0000f, 6.50000f, 2.00000f, 11.0000f, 7.50000f, 2.00000f,
                    11.0000f, 7.50000f, 2.00000f, 11.0000f, },
            { 6.50000f, 2.00000f, 11.0000f, 6.50000f, 2.00000f, 11.0000f,
                    7.50000f, 2.00000f, 11.0000f, 7.50000f, 2.00000f, 11.0000f,
                    6.50000f, 2.00000f, 11.0000f, 6.43274f, 2.93280f, 11.6403f,
                    7.32203f, 3.04107f, 11.6851f, 7.50000f, 2.00000f, 11.0000f,
                    6.08071f, 3.64575f, 12.8176f, 6.18741f, 4.14071f, 12.8601f,
                    6.91115f, 4.10281f, 12.9708f, 7.09055f, 3.64868f, 12.8302f,
                    6.00000f, 4.00000f, 14.0000f, 6.10670f, 4.49495f, 14.0424f,
                    6.82060f, 4.45412f, 14.1405f, 7.00000f, 4.00000f, 14.0000f, },
            { 6.00000f, 4.00000f, 14.0000f, 6.10670f, 4.49495f, 14.0424f,
                    6.82060f, 4.45412f, 14.1405f, 7.00000f, 4.00000f, 14.0000f,
                    5.91173f, 4.38739f, 15.2929f, 6.01843f, 4.88234f, 15.3353f,
                    6.72196f, 4.83683f, 15.4148f, 6.90135f, 4.38271f, 15.2742f,
                    5.77637f, 4.49967f, 16.7631f, 5.82299f, 4.55593f, 17.3418f,
                    6.39731f, 4.98413f, 16.8531f, 6.66387f, 4.46835f, 16.7968f,
                    6.00000f, 4.00000f, 18.0000f, 6.04662f, 4.05625f, 18.5787f,
                    6.73343f, 4.51577f, 18.0562f, 7.00000f, 4.00000f, 18.0000f, },
            { 6.00000f, 10.0000f, 18.0000f, 6.17178f, 10.3839f, 18.9501f,
                    7.00000f, 12.0000f, 20.0000f, 7.00000f, 12.0000f, 20.0000f,
                    5.82238f, 10.2143f, 15.7951f, 5.99416f, 10.5982f, 16.7453f,
                    6.36326f, 12.3748f, 19.4440f, 7.00000f, 12.0000f, 20.0000f,
                    6.99191f, 14.0339f, 16.2383f, 7.06775f, 14.0553f, 17.3163f,
                    6.99797f, 14.7084f, 18.6050f, 7.50000f, 15.0000f, 19.0000f,
                    8.00000f, 16.0000f, 16.0000f, 8.07584f, 16.0215f, 17.0779f,
                    7.50000f, 15.0000f, 19.0000f, 7.50000f, 15.0000f, 19.0000f, },
            { 6.00000f, 10.0000f, 18.0000f, 5.82238f, 10.2143f, 15.7951f,
                    6.99191f, 14.0339f, 16.2383f, 8.00000f, 16.0000f, 16.0000f,
                    5.77637f, 9.50033f, 16.7631f, 5.59875f, 9.71465f, 14.5582f,
                    6.86100f, 13.9968f, 14.3779f, 7.86909f, 15.9629f, 14.1395f,
                    5.91173f, 9.61261f, 15.2929f, 5.55206f, 11.2809f, 15.1499f,
                    6.48478f, 13.9594f, 12.5429f, 7.05435f, 15.4783f, 11.9935f,
                    6.00000f, 10.0000f, 14.0000f, 5.64033f, 11.6683f, 13.8569f,
                    5.93043f, 12.4812f, 11.5493f, 6.50000f, 14.0000f, 11.0000f, },
            { 6.00000f, 10.0000f, 14.0000f, 5.64033f, 11.6683f, 13.8569f,
                    5.93043f, 12.4812f, 11.5493f, 6.50000f, 14.0000f, 11.0000f,
                    6.00000f, 10.0000f, 14.0000f, 5.72105f, 12.0226f, 12.6746f,
                    5.71956f, 11.9188f, 11.1714f, 6.28913f, 13.4377f, 10.6220f,
                    6.00000f, 10.0000f, 14.0000f, 5.72105f, 12.0226f, 12.6746f,
                    5.95300f, 11.8801f, 11.7283f, 6.50000f, 12.0000f, 11.0000f,
                    6.00000f, 10.0000f, 14.0000f, 6.08071f, 10.3542f, 12.8176f,
                    6.50000f, 12.0000f, 11.0000f, 6.50000f, 12.0000f, 11.0000f, },
            { 7.00000f, 10.0000f, 18.0000f, 8.01538f, 11.9647f, 17.7856f,
                    9.25405f, 15.5789f, 18.1668f, 9.00000f, 16.0000f, 16.0000f,
                    7.27289f, 10.3803f, 18.9768f, 8.28828f, 12.3449f, 18.7624f,
                    9.43541f, 15.5991f, 19.2725f, 9.18135f, 16.0201f, 17.1057f,
                    8.50000f, 12.0000f, 20.0000f, 8.88419f, 12.0300f, 19.2616f,
                    9.44404f, 14.6146f, 18.6694f, 9.00000f, 15.0000f, 19.0000f,
                    8.50000f, 12.0000f, 20.0000f, 8.50000f, 12.0000f, 20.0000f,
                    9.00000f, 15.0000f, 19.0000f, 9.00000f, 15.0000f, 19.0000f, },
            { 9.00000f, 16.0000f, 16.0000f, 9.25405f, 15.5789f, 18.1668f,
                    8.01538f, 11.9647f, 17.7856f, 7.00000f, 10.0000f, 18.0000f,
                    8.69788f, 15.9664f, 14.1579f, 8.95194f, 15.5454f, 16.3247f,
                    7.67925f, 11.4963f, 16.5825f, 6.66387f, 9.53165f, 16.7968f,
                    7.84657f, 15.5366f, 12.0021f, 8.19383f, 14.9328f, 13.5421f,
                    7.49810f, 11.1280f, 14.8067f, 6.90135f, 9.61729f, 15.2742f,
                    7.50000f, 14.0000f, 11.0000f, 7.84726f, 13.3962f, 12.5400f,
                    7.59675f, 11.5107f, 13.5324f, 7.00000f, 10.0000f, 14.0000f, },
            { 7.50000f, 14.0000f, 11.0000f, 7.84726f, 13.3962f, 12.5400f,
                    7.59675f, 11.5107f, 13.5324f, 7.00000f, 10.0000f, 14.0000f,
                    7.50000f, 14.0000f, 11.0000f, 7.72215f, 12.8415f, 12.1782f,
                    7.68730f, 11.8620f, 12.3627f, 7.09055f, 10.3513f, 12.8302f,
                    7.50000f, 14.0000f, 11.0000f, 7.72215f, 12.8415f, 12.1782f,
                    7.80364f, 11.8097f, 11.7976f, 7.50000f, 12.0000f, 11.0000f,
                    7.50000f, 14.0000f, 11.0000f, 7.37488f, 13.4453f, 10.6382f,
                    7.50000f, 12.0000f, 11.0000f, 7.50000f, 12.0000f, 11.0000f, },
            { 6.00000f, 10.0000f, 18.0000f, 6.04662f, 9.94375f, 18.5787f,
                    6.73343f, 9.48423f, 18.0562f, 7.00000f, 10.0000f, 18.0000f,
                    6.17178f, 10.3839f, 18.9501f, 6.21840f, 10.3276f, 19.5288f,
                    7.00633f, 9.86448f, 19.0331f, 7.27289f, 10.3803f, 18.9768f,
                    7.00000f, 12.0000f, 20.0000f, 6.87917f, 11.3035f, 19.9349f,
                    7.96063f, 10.7283f, 19.8070f, 8.50000f, 12.0000f, 20.0000f,
                    7.00000f, 12.0000f, 20.0000f, 7.00000f, 12.0000f, 20.0000f,
                    8.50000f, 12.0000f, 20.0000f, 8.50000f, 12.0000f, 20.0000f, },
            { 7.00000f, 12.0000f, 20.0000f, 7.00000f, 12.0000f, 20.0000f,
                    8.50000f, 12.0000f, 20.0000f, 8.50000f, 12.0000f, 20.0000f,
                    7.00000f, 12.0000f, 20.0000f, 7.42866f, 13.1131f, 20.3024f,
                    8.71101f, 12.5892f, 20.1671f, 8.50000f, 12.0000f, 20.0000f,
                    7.50000f, 15.0000f, 19.0000f, 7.50022f, 14.4627f, 19.7526f,
                    8.73395f, 14.3364f, 19.9299f, 9.00000f, 15.0000f, 19.0000f,
                    7.50000f, 15.0000f, 19.0000f, 7.50000f, 15.0000f, 19.0000f,
                    9.00000f, 15.0000f, 19.0000f, 9.00000f, 15.0000f, 19.0000f, },
            { 7.50000f, 15.0000f, 19.0000f, 7.50000f, 15.0000f, 19.0000f,
                    9.00000f, 15.0000f, 19.0000f, 9.00000f, 15.0000f, 19.0000f,
                    7.50000f, 15.0000f, 19.0000f, 7.86286f, 15.9004f, 18.3200f,
                    8.92549f, 15.8374f, 18.4231f, 9.00000f, 15.0000f, 19.0000f,
                    8.07584f, 16.0215f, 17.0779f, 8.34116f, 16.5389f, 17.0151f,
                    9.11447f, 16.1310f, 16.5353f, 9.18135f, 16.0201f, 17.1057f,
                    8.00000f, 16.0000f, 16.0000f, 8.26531f, 16.5175f, 15.9372f,
                    8.93312f, 16.1108f, 15.4296f, 9.00000f, 16.0000f, 16.0000f, },
            { 8.00000f, 16.0000f, 16.0000f, 8.26531f, 16.5175f, 15.9372f,
                    8.93312f, 16.1108f, 15.4296f, 9.00000f, 16.0000f, 16.0000f,
                    7.86909f, 15.9629f, 14.1395f, 8.13441f, 16.4804f, 14.0768f,
                    8.63100f, 16.0773f, 13.5875f, 8.69788f, 15.9664f, 14.1579f,
                    7.05435f, 15.4783f, 11.9935f, 7.24070f, 15.9752f, 11.8138f,
                    7.73145f, 15.7368f, 11.4916f, 7.84657f, 15.5366f, 12.0021f,
                    6.50000f, 14.0000f, 11.0000f, 6.68634f, 14.4969f, 10.8202f,
                    7.38487f, 14.2002f, 10.4894f, 7.50000f, 14.0000f, 11.0000f, },
            { 6.50000f, 14.0000f, 11.0000f, 6.68634f, 14.4969f, 10.8202f,
                    7.38487f, 14.2002f, 10.4894f, 7.50000f, 14.0000f, 11.0000f,
                    6.28913f, 13.4377f, 10.6220f, 6.47547f, 13.9346f, 10.4423f,
                    7.25976f, 13.6455f, 10.1277f, 7.37488f, 13.4453f, 10.6382f,
                    6.50000f, 12.0000f, 11.0000f, 6.58882f, 12.5119f, 10.5862f,
                    7.51059f, 12.3798f, 10.6414f, 7.50000f, 12.0000f, 11.0000f,
                    6.50000f, 12.0000f, 11.0000f, 6.50000f, 12.0000f, 11.0000f,
                    7.50000f, 12.0000f, 11.0000f, 7.50000f, 12.0000f, 11.0000f, },
            { 6.50000f, 12.0000f, 11.0000f, 6.50000f, 12.0000f, 11.0000f,
                    7.50000f, 12.0000f, 11.0000f, 7.50000f, 12.0000f, 11.0000f,
                    6.50000f, 12.0000f, 11.0000f, 6.43274f, 11.0672f, 11.6403f,
                    7.32203f, 10.9589f, 11.6851f, 7.50000f, 12.0000f, 11.0000f,
                    6.08071f, 10.3542f, 12.8176f, 6.18741f, 9.85929f, 12.8601f,
                    6.91115f, 9.89719f, 12.9708f, 7.09055f, 10.3513f, 12.8302f,
                    6.00000f, 10.0000f, 14.0000f, 6.10670f, 9.50505f, 14.0424f,
                    6.82060f, 9.54588f, 14.1405f, 7.00000f, 10.0000f, 14.0000f, },
            { 6.00000f, 10.0000f, 14.0000f, 6.10670f, 9.50505f, 14.0424f,
                    6.82060f, 9.54588f, 14.1405f, 7.00000f, 10.0000f, 14.0000f,
                    5.91173f, 9.61261f, 15.2929f, 6.01843f, 9.11766f, 15.3353f,
                    6.72196f, 9.16317f, 15.4148f, 6.90135f, 9.61729f, 15.2742f,
                    5.77637f, 9.50033f, 16.7631f, 5.82299f, 9.44407f, 17.3418f,
                    6.39731f, 9.01587f, 16.8531f, 6.66387f, 9.53165f, 16.7968f,
                    6.00000f, 10.0000f, 18.0000f, 6.04662f, 9.94375f, 18.5787f,
                    6.73343f, 9.48423f, 18.0562f, 7.00000f, 10.0000f, 18.0000f, },
            { 6.82446f, 3.34549f, 10.8660f, 6.82446f, 3.34549f, 10.8660f,
                    5.66811f, 2.92579f, 9.10044f, 4.82907f, 2.69715f, 8.50000f,
                    6.83869f, 3.07901f, 11.1571f, 6.17186f, 2.86235f, 10.3664f,
                    5.62209f, 2.62022f, 9.33980f, 4.78305f, 2.39159f, 8.73936f,
                    6.49405f, 2.96703f, 11.4410f, 5.82862f, 2.75082f, 10.6519f,
                    5.35977f, 2.53062f, 9.79425f, 4.57038f, 2.32249f, 9.12667f,
                    6.34893f, 3.19098f, 11.7321f, 6.34893f, 3.19098f, 11.7321f,
                    5.14293f, 2.75078f, 10.0336f, 4.35354f, 2.54264f, 9.36603f, },
            { 6.34893f, 3.19098f, 11.7321f, 6.34893f, 3.19098f, 11.7321f,
                    5.14293f, 2.75078f, 10.0336f, 4.35354f, 2.54264f, 9.36603f,
                    6.07309f, 3.39608f, 11.9085f, 5.40766f, 3.17987f, 11.1194f,
                    4.92723f, 2.96978f, 10.2717f, 4.13785f, 2.76165f, 9.60412f,
                    5.93731f, 3.81398f, 11.9085f, 5.27188f, 3.59776f, 11.1194f,
                    4.75971f, 3.48534f, 10.2717f, 3.99875f, 3.18974f, 9.60412f,
                    6.03992f, 4.14204f, 11.7321f, 6.03992f, 4.14204f, 11.7321f,
                    4.80549f, 3.78930f, 10.0336f, 4.04453f, 3.49370f, 9.36603f, },
            { 6.03992f, 4.14204f, 11.7321f, 6.03992f, 4.14204f, 11.7321f,
                    4.80549f, 3.78930f, 10.0336f, 4.04453f, 3.49370f, 9.36603f,
                    6.02569f, 4.40851f, 11.4410f, 5.36025f, 4.19230f, 10.6519f,
                    4.85151f, 4.09486f, 9.79425f, 4.09055f, 3.79926f, 9.12667f,
                    6.37033f, 4.52049f, 11.1571f, 5.70350f, 4.30383f, 10.3664f,
                    5.11641f, 4.17656f, 9.33980f, 4.30322f, 3.86836f, 8.73936f,
                    6.51544f, 4.29655f, 10.8660f, 6.51544f, 4.29655f, 10.8660f,
                    5.33324f, 3.95640f, 9.10044f, 4.52006f, 3.64821f, 8.50000f, },
            { 6.51544f, 4.29655f, 10.8660f, 6.51544f, 4.29655f, 10.8660f,
                    5.33324f, 3.95640f, 9.10044f, 4.52006f, 3.64821f, 8.50000f,
                    6.79128f, 4.09145f, 10.6896f, 6.12445f, 3.87478f, 9.89891f,
                    5.54985f, 3.73649f, 8.86133f, 4.73666f, 3.42829f, 8.26090f,
                    6.92707f, 3.67355f, 10.6896f, 6.26023f, 3.45689f, 9.89891f,
                    5.71408f, 3.23103f, 8.86133f, 4.87505f, 3.00239f, 8.26090f,
                    6.82446f, 3.34549f, 10.8660f, 6.82446f, 3.34549f, 10.8660f,
                    5.66811f, 2.92579f, 9.10044f, 4.82907f, 2.69715f, 8.50000f, },
            { 4.82907f, 2.69715f, 8.50000f, 4.01638f, 2.47569f, 7.91842f,
                    2.04314f, 2.05481f, 7.43301f, 2.04314f, 2.05481f, 7.43301f,
                    4.78305f, 2.39159f, 8.73936f, 3.97036f, 2.17013f, 8.15778f,
                    2.94601f, 2.05055f, 7.80752f, 2.04314f, 2.05481f, 7.43301f,
                    4.57038f, 2.32249f, 9.12667f, 3.81921f, 2.12442f, 8.49139f,
                    2.70969f, 1.96031f, 8.29323f, 1.80537f, 1.97756f, 7.86603f,
                    4.35354f, 2.54264f, 9.36603f, 3.60237f, 2.34458f, 8.73075f,
                    1.80537f, 1.97756f, 7.86603f, 1.80537f, 1.97756f, 7.86603f, },
            { 4.35354f, 2.54264f, 9.36603f, 3.60237f, 2.34458f, 8.73075f,
                    1.80537f, 1.97756f, 7.86603f, 1.80537f, 1.97756f, 7.86603f,
                    4.13785f, 2.76165f, 9.60412f, 3.38667f, 2.56358f, 8.96885f,
                    2.49788f, 2.17525f, 8.52685f, 1.80537f, 1.97756f, 7.86603f,
                    3.99875f, 3.18974f, 9.60412f, 3.27462f, 2.90844f, 8.96886f,
                    2.32732f, 2.70019f, 8.52685f, 1.65086f, 2.45308f, 7.86603f,
                    4.04453f, 3.49370f, 9.36603f, 3.32040f, 3.21241f, 8.73076f,
                    1.65086f, 2.45308f, 7.86603f, 1.65086f, 2.45308f, 7.86603f, },
            { 4.04453f, 3.49370f, 9.36603f, 3.32040f, 3.21241f, 8.73076f,
                    1.65086f, 2.45308f, 7.86603f, 1.65086f, 2.45308f, 7.86603f,
                    4.09055f, 3.79926f, 9.12667f, 3.36642f, 3.51797f, 8.49140f,
                    2.37234f, 2.99857f, 8.29323f, 1.65086f, 2.45308f, 7.86603f,
                    4.30322f, 3.86836f, 8.73936f, 3.51557f, 3.56984f, 8.15778f,
                    2.61656f, 3.06448f, 7.80752f, 1.88863f, 2.53034f, 7.43301f,
                    4.52006f, 3.64821f, 8.50000f, 3.73241f, 3.34968f, 7.91842f,
                    1.88863f, 2.53034f, 7.43301f, 1.88863f, 2.53034f, 7.43301f, },
            { 4.52006f, 3.64821f, 8.50000f, 3.73241f, 3.34968f, 7.91842f,
                    1.88863f, 2.53034f, 7.43301f, 1.88863f, 2.53034f, 7.43301f,
                    4.73666f, 3.42829f, 8.26090f, 3.94901f, 3.12976f, 7.67932f,
                    2.82260f, 2.85571f, 7.58047f, 1.88863f, 2.53034f, 7.43301f,
                    4.87505f, 3.00239f, 8.26090f, 4.06236f, 2.78093f, 7.67932f,
                    2.98998f, 2.34056f, 7.58047f, 2.04314f, 2.05481f, 7.43301f,
                    4.82907f, 2.69715f, 8.50000f, 4.01638f, 2.47569f, 7.91842f,
                    2.04314f, 2.05481f, 7.43301f, 2.04314f, 2.05481f, 7.43301f, },
            { 1.80537f, 1.97756f, 7.86603f, 1.80537f, 1.97756f, 7.86603f,
                    2.04314f, 2.05481f, 7.43301f, 2.04314f, 2.05481f, 7.43301f,
                    1.80537f, 1.97756f, 7.86603f, 1.75245f, 1.96950f, 7.82668f,
                    1.94375f, 2.04371f, 7.40157f, 2.04314f, 2.05481f, 7.43301f,
                    1.65086f, 2.45308f, 7.86603f, 1.60331f, 2.42850f, 7.82668f,
                    1.80169f, 2.48090f, 7.40157f, 1.88863f, 2.53034f, 7.43301f,
                    1.65086f, 2.45308f, 7.86603f, 1.65086f, 2.45308f, 7.86603f,
                    1.88863f, 2.53034f, 7.43301f, 1.88863f, 2.53034f, 7.43301f, },
            { 6.82446f, 9.15451f, 10.8660f, 6.82446f, 9.15451f, 10.8660f,
                    5.64226f, 9.49465f, 9.10044f, 4.82907f, 9.80285f, 8.50000f,
                    6.67934f, 8.93056f, 11.1571f, 6.01251f, 9.14723f, 10.3664f,
                    5.42542f, 9.27449f, 9.33980f, 4.61223f, 9.58269f, 8.73936f,
                    6.33470f, 9.04254f, 11.4410f, 5.66927f, 9.25875f, 10.6519f,
                    5.16053f, 9.35619f, 9.79425f, 4.39957f, 9.65179f, 9.12667f,
                    6.34893f, 9.30902f, 11.7321f, 6.34893f, 9.30902f, 11.7321f,
                    5.11451f, 9.66176f, 10.0336f, 4.35354f, 9.95736f, 9.36603f, },
            { 6.34893f, 9.30902f, 11.7321f, 6.34893f, 9.30902f, 11.7321f,
                    5.11451f, 9.66176f, 10.0336f, 4.35354f, 9.95736f, 9.36603f,
                    6.24633f, 9.63708f, 11.9085f, 5.58090f, 9.85329f, 11.1194f,
                    5.06873f, 9.96572f, 10.2717f, 4.30777f, 10.2613f, 9.60412f,
                    6.38211f, 10.0550f, 11.9085f, 5.71668f, 10.2712f, 11.1194f,
                    5.23624f, 10.4813f, 10.2717f, 4.44686f, 10.6894f, 9.60412f,
                    6.65795f, 10.2601f, 11.7321f, 6.65795f, 10.2601f, 11.7321f,
                    5.45194f, 10.7003f, 10.0336f, 4.66256f, 10.9084f, 9.36603f, },
            { 6.65795f, 10.2601f, 11.7321f, 6.65795f, 10.2601f, 11.7321f,
                    5.45194f, 10.7003f, 10.0336f, 4.66256f, 10.9084f, 9.36603f,
                    6.80307f, 10.4840f, 11.4410f, 6.13764f, 10.7002f, 10.6519f,
                    5.66878f, 10.9204f, 9.79425f, 4.87940f, 11.1286f, 9.12667f,
                    7.14771f, 10.3720f, 11.1571f, 6.48088f, 10.5887f, 10.3664f,
                    5.93110f, 10.8308f, 9.33980f, 5.09207f, 11.0595f, 8.73936f,
                    7.13348f, 10.1056f, 10.8660f, 7.13348f, 10.1056f, 10.8660f,
                    5.97713f, 10.5253f, 9.10044f, 5.13809f, 10.7539f, 8.50000f, },
            { 7.13348f, 10.1056f, 10.8660f, 7.13348f, 10.1056f, 10.8660f,
                    5.97713f, 10.5253f, 9.10044f, 5.13809f, 10.7539f, 8.50000f,
                    7.23608f, 9.77750f, 10.6896f, 6.56925f, 9.99417f, 9.89891f,
                    6.02310f, 10.2200f, 8.86133f, 5.18406f, 10.4487f, 8.26090f,
                    7.10030f, 9.35961f, 10.6896f, 6.43347f, 9.57627f, 9.89891f,
                    5.85887f, 9.71457f, 8.86133f, 5.04568f, 10.0228f, 8.26090f,
                    6.82446f, 9.15451f, 10.8660f, 6.82446f, 9.15451f, 10.8660f,
                    5.64226f, 9.49465f, 9.10044f, 4.82907f, 9.80285f, 8.50000f, },
            { 4.82907f, 9.80285f, 8.50000f, 4.04142f, 10.1014f, 7.91842f,
                    2.19765f, 10.9207f, 7.43301f, 2.19765f, 10.9207f, 7.43301f,
                    4.61223f, 9.58269f, 8.73936f, 3.82458f, 9.88122f, 8.15778f,
                    2.92558f, 10.3866f, 7.80752f, 2.19765f, 10.9207f, 7.43301f,
                    4.39957f, 9.65179f, 9.12667f, 3.67543f, 9.93309f, 8.49139f,
                    2.68136f, 10.4525f, 8.29323f, 1.95988f, 10.9980f, 7.86603f,
                    4.35354f, 9.95736f, 9.36603f, 3.62941f, 10.2387f, 8.73075f,
                    1.95988f, 10.9980f, 7.86603f, 1.95988f, 10.9980f, 7.86603f, },
            { 4.35354f, 9.95736f, 9.36603f, 3.62941f, 10.2387f, 8.73075f,
                    1.95988f, 10.9980f, 7.86603f, 1.95988f, 10.9980f, 7.86603f,
                    4.30777f, 10.2613f, 9.60412f, 3.58363f, 10.5426f, 8.96885f,
                    2.63633f, 10.7509f, 8.52685f, 1.95988f, 10.9980f, 7.86603f,
                    4.44686f, 10.6894f, 9.60412f, 3.69569f, 10.8875f, 8.96886f,
                    2.80690f, 11.2758f, 8.52685f, 2.11439f, 11.4735f, 7.86603f,
                    4.66256f, 10.9084f, 9.36603f, 3.91139f, 11.1065f, 8.73076f,
                    2.11439f, 11.4735f, 7.86603f, 2.11439f, 11.4735f, 7.86603f, },
            { 4.66256f, 10.9084f, 9.36603f, 3.91139f, 11.1065f, 8.73076f,
                    2.11439f, 11.4735f, 7.86603f, 2.11439f, 11.4735f, 7.86603f,
                    4.87940f, 11.1286f, 9.12667f, 4.12823f, 11.3266f, 8.49140f,
                    3.01871f, 11.4907f, 8.29323f, 2.11439f, 11.4735f, 7.86603f,
                    5.09207f, 11.0595f, 8.73936f, 4.27938f, 11.2809f, 8.15778f,
                    3.25502f, 11.4005f, 7.80752f, 2.35215f, 11.3962f, 7.43301f,
                    5.13809f, 10.7539f, 8.50000f, 4.32540f, 10.9754f, 7.91842f,
                    2.35215f, 11.3962f, 7.43301f, 2.35215f, 11.3962f, 7.43301f, },
            { 5.13809f, 10.7539f, 8.50000f, 4.32540f, 10.9754f, 7.91842f,
                    2.35215f, 11.3962f, 7.43301f, 2.35215f, 11.3962f, 7.43301f,
                    5.18406f, 10.4487f, 8.26090f, 4.37137f, 10.6701f, 7.67932f,
                    3.29900f, 11.1105f, 7.58047f, 2.35215f, 11.3962f, 7.43301f,
                    5.04568f, 10.0228f, 8.26090f, 4.25803f, 10.3213f, 7.67932f,
                    3.13162f, 10.5954f, 7.58047f, 2.19765f, 10.9207f, 7.43301f,
                    4.82907f, 9.80285f, 8.50000f, 4.04142f, 10.1014f, 7.91842f,
                    2.19765f, 10.9207f, 7.43301f, 2.19765f, 10.9207f, 7.43301f, },
            { 1.95988f, 10.9980f, 7.86603f, 1.95988f, 10.9980f, 7.86603f,
                    2.19765f, 10.9207f, 7.43301f, 2.19765f, 10.9207f, 7.43301f,
                    1.95988f, 10.9980f, 7.86603f, 1.91233f, 11.0226f, 7.82668f,
                    2.11071f, 10.9702f, 7.40157f, 2.19765f, 10.9207f, 7.43301f,
                    2.11439f, 11.4735f, 7.86603f, 2.06147f, 11.4816f, 7.82668f,
                    2.25276f, 11.4074f, 7.40157f, 2.35215f, 11.3962f, 7.43301f,
                    2.11439f, 11.4735f, 7.86603f, 2.11439f, 11.4735f, 7.86603f,
                    2.35215f, 11.3962f, 7.43301f, 2.35215f, 11.3962f, 7.43301f, }, };

    public Gumbo() {
        super(PATCHES);
    }
}
